import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.URL;

/**
 * BrowserTest osztly - HTML bngsz program
 *                        hipermdia kpessgeit szemlltet applet
 * @see java.applet.Applet
 */
public class BrowserTest extends Applet
                         implements Runnable, ActionListener, MouseListener {
  String imageSource=null;                             //kpek knyvtrnak neve
  String soundSource=null;                            //hangok knyvtrnak neve
  SoundList soundList;                                    //hangbetlts vezrl
  Image background;                                                  //httrkp
  AudioClip sound;                                                  //httrzene
  String soundName;                                  //httr hanganyag fjlneve
  private boolean loaded;                     //paramter-feldolgozs befejezve?
  private int count=1;                                       //gombok szmllja
  private Thread kicker;                                          //sajt Thread
  int statusDelay=100;                    //sttuszsor grgetsnek ksleltetse
  String statusText=null;                                    //grgetend szveg
  private StringBuffer displayText=new StringBuffer();  //aktulis sttuszszveg

  /**
   * Sznparamter feldolgozsa
   * @param szin a sznt reprezentl szveg
   * @result a krt szn, vagy null, ha a paramter helytelen
   */
  Color decodeColor(String szin) {
    int val=0;
    try {
        if (szin.startsWith("0x")) {               //hexadecimlisan van megadva
           val=Integer.parseInt(szin.substring(2), 16);
        } else if (szin.startsWith("#")) {         //hexadecimlisan van megadva
               val=Integer.parseInt(szin.substring(1), 16);
        } else if (szin.startsWith("0") && szin.length()>1) {          //oktlis
               val=Integer.parseInt(szin.substring(1), 8);
        } else val=Integer.parseInt(szin, 10);         //decimlisan van megadva
        return new Color(val);
    } catch (NumberFormatException e) {
      return null;
    }
  }
  
  /**
   * Paramterekrl informci
   */
  public String[][] getParameterInfo() {
    String info[][]={
       {"foregroundcolor", "szn", "alaprtelmezett eltrszn"},
       {"backgroundcolor", "szn", "alaprtelmezett httrszn"},
       {"imagesource", "URL", "alaprtelmezett relatv URL a kpekhez"},
       {"soundsource", "URL", "alaprtelmezett relatv URL a hangokhoz"},
       {"background", "kpfjl", "httrkp"},
       {"sound", "hangfjl", "httrzene"},
       {"statusdelay", "int", "grgets vrakozsi ideje sec-ban"},
       {"textX", "szveg", "X. gomb felirata"},
       {"imageX", "kpfjl", "X. gomb kpe"},
       {"soundX", "hangfjl", "X. gomb httrzenje"},
       {"clicksoundX", "hangfjl", "X. gomb kivlasztskori zenje"},
       {"foregroundcolorX", "szn", "X. gomb eltrszne"},
       {"backgroundcolorX", "szn", "X. gomb httrszne"},
       {"linkX", "URL", "X. gomb megjelentend linkje"},
       {"rellinkX", "URL", "X. gomb megjelentend relatv linkje"},
       {"linkframeX", "frame", "X. gomb linkjnek frame-je"},
       {"statustextX", "szveg", "X. gomb grgetett sttuszszvege"}
    };
    return info;
  }
  
  /**
   * Appletrl informci
   */
  public String getAppletInfo() {
    return "Az applet a bngsz program hipermdia kpessgeit szemllteti.";
  }
  
  public void init() {                                           //inicializls
    Color color;

    String param=getParameter("imageSource");    //globlis paramterek tvtele
    try {
        imageSource=(param==null) ? "" : (param+"/");
    } catch (Exception e) {
      System.err.println("BrowserTest imageSource: "+e+" hibs."+
                         "Helyette a DocumentBase lesz figyelembe vve");
      imageSource="";
    }
    
    param=getParameter("soundSource");                             //imagesource
    try {
        soundSource=(param==null) ? "" : (param+"/");
    } catch (Exception e) {
      System.err.println("BrowserTest soundSource: "+e+" hibs."+
                         "Helyette a DocumentBase lesz figyelembe vve");
      soundSource="";
    }
    
    if ((param=getParameter("background"))!=null) {                 //background
       try {
           background=getImage(getClass().getResource(imageSource+param));
       } catch (Exception e) {
         System.err.println(this+": "+e);
       }
    }
    
    if ((param=getParameter("backgroundColor"))!=null &&       //backgroundcolor
       ((color=decodeColor(param))!=null))
       setBackground(color);
    if ((param=getParameter("foregroundColor"))!=null &&       //foregroundcolor
       ((color=decodeColor(param))!=null))
       setForeground(color);

    soundList=new SoundList(this, soundSource);
    if ((soundName=getParameter("sound"))!=null)                         //sound
       soundList.startLoading(soundName);                     //zenetlts indul

    if ((param=getParameter("statusDelay"))!=null) {               //statusdelay
       try {
           statusDelay=Integer.parseInt(param);
       } catch (Exception e) {}
    }
    if (statusDelay<100) statusDelay=100;
    
    addMouseListener(this);
  }
  
  public void start() {                                                //indts
    if (kicker==null) {                                  //sajt Thread indtsa
       kicker=new Thread(this);
       kicker.setPriority(Thread.MIN_PRIORITY+1);
       kicker.start();
    }
  }
  
  public void run() {                      //sajt programszlbl hvott metdus
    String text;
    String image;
    String param, param2;
    Image img;
    GraphicsButton gb;
    Color color;

    if (!loaded) {                                    //mg paramterfeldolgozs
       while (((text=getParameter("text"+count))!=null) |
            ((image=getParameter("image"+count))!=null)) {
            img=null;
            if (image!=null)                                  //kp feldolgozsa
               try {
                   prepareImage(img=getImage( getClass().getResource(
                                 imageSource+image)), this);
                   while (((checkImage(img, this) & ERROR)!=ERROR) &&//vrakozs
                        ((checkImage(img,this)&(WIDTH|HEIGHT))!=(WIDTH|HEIGHT)))
                        try {
                            Thread.currentThread().sleep(100);
                        } catch (Exception e) {}
               } catch (Exception e) {}
            gb=new GraphicsButton(img, text);        //j gomb hozzvtele

            if ((param=getParameter("link"+count))!=null)                //linkX
               try {
                   gb.setLink(new URL(param));
               } catch (Exception e) {
                 System.err.println(this+" link: "+e);
               }
            if ((param=getParameter("rellink"+count))!=null)          //rellinkX
               try {
                   gb.setLink(new URL(getDocumentBase(), param));
               } catch (Exception e) {
                 System.err.println(this+" rellink: "+e);
               }
            gb.setFrame(getParameter("linkframe"+count));           //linkframeX
            
            gb.setStatusText(getParameter("statustext"+count));    //statustextX

            param=getParameter("sound"+count);                          //soundX
            param2=getParameter("clicksound"+count);               //clicksoundX
            gb.setSounds(param, param2, soundList);
            
            color=null;
            if ((param=getParameter("backgroundColor"+count))!=null)
               color=decodeColor(param);                      //backgroundcolorX
            if (color==null) color=getBackground();
            gb.setBackground(color);
            color=null;
            if ((param=getParameter("foregroundColor"+count))!=null)
               color=decodeColor(param);                      //foregroundcolorX
            if (color==null) color=getForeground();
            gb.setForeground(color);
            
            add(gb);
            gb.addActionListener(this);
            gb.addMouseListener(this);
            validate();
            transferFocus();
            count++;                       //jhet a kvetkez gomb feldolgozsa
            if (Thread.currentThread()!=kicker) {       //thread le lett lltva
               return;
            }
        }
        loaded=true;                            //paramterfeldolgozs befejezve
    }
    displayText.setLength(0);

    while (Thread.currentThread()==kicker) {            //amg aktulis a Thread
          scroll();                                                   //grgets
          try {
              Thread.currentThread().sleep(statusDelay);             //vrakozs
          } catch (Exception e) {}
    }
  }
  
  private synchronized void scroll() {                  //sttuszsorban grgets
    if (statusText!=null) {                                   //van mit grgetni
       if (displayText.length()==0) {                            //mg nem grg
          displayText.ensureCapacity(statusText.length());
          displayText.setLength(0);
          displayText.insert(0, statusText);
       }
       
       showStatus(displayText.toString());                              //kirs

       if (displayText.length()>0) {                                  //lptets
          for (int i=1; i<displayText.length(); i++)
             displayText.setCharAt(i-1,displayText.charAt(i));
          displayText.setLength(displayText.length()-1);
       }
    }
  }

  public void stop() {                                              //megllts
       if (kicker!=null) kicker.interrupt();          //sajt Thread meglltsa
       kicker=null;
  }

  /**
   * j szveg grgetse
   */
  public synchronized void scrollStatus(String text) {
    statusText=text;
    showStatus("");
    displayText.setLength(0);
  }

  public void update(Graphics g) {
      paint(g);
  }

  public void paint(Graphics g) {                                   //kirajzols
    if (background!=null)                               //httrkp megrajzolsa
       g.drawImage(background, 0, 0, getSize().width, getSize().height, this);
  }

  public void mouseEntered(MouseEvent evt) {                      //kurzor belp
    if (evt.getSource() instanceof GraphicsButton){//gombok szvegnek grgetse
       scrollStatus(((GraphicsButton)evt.getSource()).getStatusText());
       return;
    }
    if (sound==null && soundName!=null)                   //httrzene betltve?
       sound=soundList.getClip(soundName);
    if (sound!=null) sound.loop();                         //httrzene indtsa
  }

  public void mouseExited(MouseEvent evt) {                       //kurzor kilp
    scrollStatus(null);
    if (evt.getSource()==this&&sound!=null) sound.stop();//httrzene lelltsa
  }

  public void mousePressed(MouseEvent evt) {};                   //MouseListener
  public void mouseReleased(MouseEvent evt) {};                      //interfsz
  public void mouseClicked(MouseEvent evt) {};                  //miatt kellenek

  public void actionPerformed(ActionEvent evt){                  //esemnykezel
    GraphicsButton gb=(GraphicsButton)evt.getSource();
    if (gb.getLink()!=null) {                         //adott link megjelentse
       if (gb.getFrame()==null) getAppletContext().showDocument(gb.getLink());
       else getAppletContext().showDocument(gb.getLink(), gb.getFrame());
    }
  }
}
