import java.awt.*;
import java.awt.event.*;
import java.applet.AudioClip;
import java.net.URL;

/**
 * GraphicsButton osztly - kpet is megjelent multimdia nyomgomb
 * @see java.awt.Canvas
 */
class GraphicsButton extends Button
          implements FocusListener, KeyListener, ActionListener, MouseListener {
  private boolean pressed=false;                                 //be van nyomva
  Image picture;                                                     //httrkp
  String text;                                                         //felirat
  URL link;                                                //megjelentend link
  String linkFrame;                                      //megjelents frame-je
  String statusText;                                         //grgetend szveg
  String soundName;                                        //httrzene fjlneve
  String clickSoundName;                            //kivlasztsi zene fjlneve
  SoundList soundList;                                  //zenebetlts vezrlje
  AudioClip sound;                                                  //httrzene
  AudioClip clickSound;                                      //kivlasztsi zene
  Image offscreen;                                               //offscreen kp

  /**
   * Grafikus nyomgomb ltrehozsa
   * @param img a gomb httrkpe
   * @param text a gomb felirata
   */
  public GraphicsButton(Image img, String text) {
    super();
    picture=img;
    this.text=text;
    addFocusListener(this);                                //esemnyek figyelse
    addKeyListener(this);
    addMouseListener(this);
    addActionListener(this);
    requestFocus();
  }

  public URL getLink() {                          //lekrdez-bellt metdusok
    return link;
  }
  
  public void setLink(URL link) {
    this.link=link;
  }
  
  public String getFrame() {
    return linkFrame;
  }
  
  public void setFrame(String frame) {
    linkFrame=frame;
  }
  
  public String getStatusText() {
    return statusText;
  }

  public void setStatusText(String text) {
    statusText=text;
  }

  public void setSounds(String soundName, String clickSoundName,
                        SoundList soundList) {
    this.soundName=soundName;
    this.clickSoundName=clickSoundName;
    this.soundList=soundList;
    if (soundName!=null)
       soundList.startLoading(soundName);                   
    if (clickSoundName!=null)
       soundList.startLoading(clickSoundName);                   
  }

  public void invalidate() {
    super.invalidate();
    offscreen=null;                                  //offscreen buffer eldobsa
  }

  public Dimension getPreferredSize() {                  //sajt mret megrzse
    if (picture!=null &&                    //ha van httrkp, akkor van mrete
       (checkImage(picture, this)&(WIDTH|HEIGHT))==(WIDTH|HEIGHT))
       return new Dimension(
                  picture.getWidth(this)+10, picture.getHeight(this)+10);
    Font f=getFont();
    if (f!=null) {
       int w=0;
       FontMetrics fm = getFontMetrics(f);
       if (text!=null) w=fm.stringWidth(text);
       return new Dimension(w+10, fm.getHeight()+fm.getAscent()+5);
    }
    return getMinimumSize();
  }
  
  public Dimension getMinimumSize() {                    //alaprtelmezett mret
    return new Dimension(10, 10);
  }
  
  public void update(Graphics g) {                                  //kirajzols
    paint(g);
  }
  
  public void paint(Graphics g) {                                   //kirajzols
    if (getSize().width==0 || getSize().height==0) return;
    int sx, sy;
    if (offscreen==null)                          //offscreen buffer ltrehozsa
       offscreen=createImage(getSize().width, getSize().height);
    Graphics og=offscreen.getGraphics();                    //offscreen rajzols
  
    if (hasFocus()) og.setColor(getForeground());          //aktulis kr keret
    else og.setColor(getBackground());
    og.fillRect(0, 0, getSize().width, getSize().height);
    og.setColor(getForeground());
    if (pressed) sx=sy=1;
    else sx=sy=0;
  
    if ((picture!=null)&&((checkImage(picture, this)&ERROR)==ERROR))
       picture=null;                                 //httrkp nem tlthet be
    og.draw3DRect(2, 2, getSize().width-5, getSize().height-5, !pressed);
    og.setColor(getBackground());
    og.fill3DRect(3, 3, getSize().width-6, getSize().height-6, !pressed);
    if (picture!=null)                                   //httrkp kirajzolsa
      og.drawImage(picture,sx+3,sy+3,getSize().width-6,getSize().height-6,this);

    if (text!=null) {                                //felirat kzpre rajzolsa
       FontMetrics fm=g.getFontMetrics();
       if (fm!=null) {
          og.setColor(getForeground());
          sx+=(int)(getSize().width-fm.stringWidth(text))/2;
          sy+=(int)(getSize().height-fm.getHeight())/2+fm.getAscent();
          og.drawString(text, sx, sy);
       }
    }
    g.drawImage(offscreen, 0, 0, null);              //offscreen kp kirajzolsa
    og.dispose();
  }

  public void actionPerformed(ActionEvent e) {            //megnyoms lekezelse
    if (clickSound==null && clickSoundName!=null)          //zene be van tltve?
       clickSound=soundList.getClip(clickSoundName);
    if (clickSound!=null) clickSound.play();                           //zenls

    pressed=false;
    repaint();
  }

  public void focusGained(FocusEvent e) {                   //megkapta a fkuszt
    repaint();
  }
  public void focusLost(FocusEvent e) {                  //elvesztette a fkuszt
    pressed=false;
    repaint();
  }

  public void keyReleased(KeyEvent e) {}
  public void keyTyped(KeyEvent e) {};
  public void keyPressed(KeyEvent e) {
    if ((e.getKeyCode()==KeyEvent.VK_SPACE)&&!pressed) {  //megnyomtk a szkzt
       pressed=true;
       repaint();
    }
  }

  public void mouseEntered(MouseEvent e) {                        //kurzor belp
    if (sound==null && soundName!=null)                   //httrzene betltve?
       sound=soundList.getClip(soundName);
    if (sound!=null) sound.loop();                               //httrzenls
  }

  public void mouseExited(MouseEvent e) {                         //kurzor kilp
    if (pressed) {                                          //mr nincs benyomva
       pressed=false;
       repaint();
    }
    if (sound!=null) sound.stop();                            //httrzene lell
  }

  public void mouseClicked(MouseEvent e) {};
  public void mousePressed(MouseEvent e) {                  //egrgomb megnyomva
    if (!pressed) {
       pressed=true;
       requestFocus();
       repaint();
    }
  }

  public void mouseReleased(MouseEvent e) {
    if (pressed) {
       pressed=false;
       repaint();
    }
  }
}
