package javabook.smiley;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.beans.*;

// An IntBox Bean is a TextField that only accepts positive integers.
public class IntBox2 extends TextField implements Serializable
{
    // The previous value of the IntBox.
    int prev_ = 0;

    // Delegate property change operations to this object    
    PropertyChangeSupport pcs_ = new PropertyChangeSupport(this);  

    // Default constructor.
    public IntBox2()
    {
        super();
        enableEvents(java.awt.AWTEvent.KEY_EVENT_MASK);
//        pcs_ = new PropertyChangeSupport(this);
//        prev_ = 0;
    }

    // Allow other objects to register interest in my bound properties
    public void addPropertyChangeListener(PropertyChangeListener listener)
    {
        pcs_.addPropertyChangeListener(listener);
    }

    // Allow other objects to remove their interest in my bound properties
    public void removePropertyChangeListener(PropertyChangeListener listener)
    {
        pcs_.removePropertyChangeListener(listener);
    }

    // Return the integer value of this object.
    // It's zero if we can't parse it.
    public int getValue()
    {
        int i;
        try {
            i = Integer.parseInt(getText());
            return i;
        }
        catch (NumberFormatException ex) {
            return 0;
        }
    }
    
    // Set new value
    public void setValue(int value)
    {
        int oldValue = prev_;
        setText(Integer.toString(value));
        int newValue = getValue();

        // This call sends property change events to all listeners.
        pcs_.firePropertyChange("value",
                                new Integer(oldValue),
                                new Integer(newValue));
        prev_ = newValue;
    }

    // Discard non-numeric, non-backspace characters.
    public void processKeyEvent(KeyEvent event)
    {
        char c = event.getKeyChar();

        // These events get default processing.
        if ((c >= '0' && c <= '9') ||
            c == KeyEvent.VK_DELETE ||
            c == KeyEvent.VK_BACK_SPACE) {
            return;
        }

        // This event gets default processing, but the setValue()
        // accessor is also called.
        if (c == KeyEvent.VK_ENTER) {
            int i = getValue();
            setValue(i);
            return;
        }

        // All other characters we discard
        event.consume();
    }
}
