/**
 * Egy egyszer bean kttt tulajdonsgokkal s egy vtzhatval.
 * A vtzhat tulajdonsg a "funPercent".  VetoablePropertyChange 
 * figyelk elutasthatjk a tulajdonsg javasolt vltozst egy
 * PropertyVetoException kivtel eldobsval.
 * 
 */
package javabook.smiley;

import java.awt.*;
import java.util.*;
import java.beans.*;

public class SmileyBean extends Component 
                        implements PropertyChangeListener {

    /**
     * Konstruktor
     */
    public SmileyBean() {
    }

    /**
     * Kirajzolja a mosolyg arcot
     */
    public void paint(Graphics g) {
       g.setColor( faceColor );
       g.fillArc( 5, 5, 40, 40, 0, 360 );
       g.setColor( drawColor );
       g.drawArc( 5, 5, 40, 40, 0, 360 );
       g.fillArc( 15, 18, 5, 5, 0, 360 );
       g.fillArc( 30, 18, 5, 5, 0, 360 );
       int a = ( int )( 0.5 + ( ourFunPercent / 100.0 ) * MAX_SMILEY_ANGLE );
       if ( a < 4 ) { a = 0; } // Hogy az v ne legyen kisebb, mint egy pixel
       g.drawArc( 15, 15, 20, 20, -( 180 - a ) / 2, -a );
    }

    /**
     * A propertyChangeEvent kezelje, az esemny alapjn belltja 
     * a funPercent j rtkt
     */
    public void propertyChange( PropertyChangeEvent event ) {
       Integer newValue = ( Integer )event.getNewValue();
       try {
          setFunPercent( newValue.intValue() );
       } catch ( PropertyVetoException e ) {
       }
    }
    
    public Dimension getPreferredSize() {
       return new Dimension(50,50);
    }

    /** 
     * Visszatr rajzolat sznvel
     */
    public synchronized Color getDrawColor() {
        return drawColor;
    }

    /** 
     * Belltja a rajzolat sznt. Ez egy kttt tulajdonsg
     */
    public void setDrawColor( Color newColor ) {
       Color oldColor = drawColor;
       drawColor = newColor;
       changes.firePropertyChange( "drawColor", oldColor, newColor );
       repaint();
    }

    /** 
     * Visszatr az arc sznvel
     */
    public synchronized Color getFaceColor() {
        return faceColor;
    }

    /** 
     * Belltja az arc sznt. Ez egy kttt tulajdonsg
     */
    public void setFaceColor( Color newColor ) {
       Color oldColor = faceColor;
       faceColor = newColor;
       changes.firePropertyChange( "faceColor", oldColor, newColor );
       repaint();
    }

    /** 
     * Visszatr a minimlis rtestend vltozs rtkvel
     */
    public synchronized int getMinChangeToNotify() {
        return minChangeToNotify;
    }

    /** 
     * Belltja a minimlis rtestend vltozs rtkt.
     * Ez egy kttt tulajdonsg
     */
    public void setMinChangeToNotify( int newMinChangeToNotify ) {
       int oldMinChangeToNotify = minChangeToNotify;
       minChangeToNotify = newMinChangeToNotify;
       changes.firePropertyChange( "minChangeToNotify", 
                                   oldMinChangeToNotify, 
                                   newMinChangeToNotify );
    }

    /** 
     * Visszatr az aktulisan belltott sznsmval
     */
    public synchronized String getColorScheme() {
        return colorScheme;
    }

    /** 
     * Belltja az aktulis sznsmt
     */
    public void setColorScheme( String scheme ) {
       if ( scheme.equals( COL_GRAY ) ) {
          setFaceColor( Color.lightGray );
          setDrawColor( Color.darkGray );
          colorScheme = scheme;
       }
       else if ( scheme.equals( COL_ATROCIOUS ) ) {
          setFaceColor( new Color( 10, 75, 10 ) );
          setDrawColor( new Color( 255, 128, 128 ) );
          colorScheme = scheme;
       }
       else if ( scheme.equals( COL_HIDEOUS ) ) {
          setFaceColor( new Color( 255, 200, 0 ) );
          setDrawColor( new Color( 255, 95, 255 ) );
          colorScheme = scheme;
       }
       else {
          // Egybknt legyen alaprtelmezett
          scheme = COL_DEFAULT;
       }
       
       if ( scheme.equals( COL_DEFAULT ) ) {
          setFaceColor( Color.yellow );
          setDrawColor( Color.black );
          colorScheme = scheme;
       }
    }

    /** 
     * Visszaadja az aktulis vidmsg fokozatot szzalkban
     */
    public synchronized int getFunPercent() {
        return ourFunPercent;
    }

    /**
     * Belltja a vidmsg fokozatot szzalkban, hacsak nem dob el egy 
     * regisztrlt VetoableChangeListener egy PropertyVetoException kivtelt.
     * Ez egy vtozhat kttt tulajdonsg.
     */
    public void setFunPercent( int newFunPercent )
                            throws PropertyVetoException {
       if ( newFunPercent <= 100 && newFunPercent >= 0 ) {
         int oldFunPercent = ourFunPercent;

         // Elszr rtestjk a vtzkat a vltozsrl. Ha brmelyik kivtelt dob
         // nem kapjuk el, hanem a fggvny hvjnak delegljuk tovbb.
         vetos.fireVetoableChange("funPercent", 
                    new Integer( oldFunPercent ),
                    new Integer( newFunPercent ) );
         // Ha senki nem vtzott, menjnk tovbb s vltoztassuk meg az rtket
         ourFunPercent = newFunPercent;
         
         if ( oldFunPercent != newFunPercent ) { repaint(); }
         
         changes.firePropertyChange("funPercent", 
                      new Integer( oldFunPercent ),
                      new Integer( newFunPercent ) );
                      
         // rtesti a regisztrlt SmileyListener figyelket, ha a vltozs
         // meghaladja a minChangeToNotify rtket
         if ( Math.abs( oldFunPercent - newFunPercent ) >= minChangeToNotify ) {
            notifySmileyChanged( oldFunPercent, newFunPercent );
         }

       }
    }
    
    //--------------------------------------------------------------------------
    // A kvetkez metdusok a SmileyListener-eket kezelik

    /**
     * Hozzad egy SmileyListenert
     */
    public synchronized void addSmileyListener( SmileyListener l ) {
       smileyListeners.addElement( l );
    }
   
    /**
     * Kitrli a regisztrcis listrl a SmileyListenert
     */
    public synchronized void removeSmileyListener( SmileyListener l ) {
       smileyListeners.removeElement( l );
    }

    /**
     * rtesti a figyelket, ha vltozs trtnt
     */
    protected void notifySmileyChanged( int oldFunPercent, int newFunPercent ) {
       Vector thisList = new Vector();
       
       SmileyEvent thisEvent = new SmileyEvent( this, 
                                                oldFunPercent, 
                                                newFunPercent );
       // Ksztsnk msolatot a listrl, hogy ms szlak ltal lehetsges
       // vltozsai ne tegyenek keresztbe
       synchronized ( this ) {
          thisList = ( Vector )smileyListeners.clone();
       }
      
       // Kldjk el az esemnyt minden regisztrlt figyelnek
       for ( int i = 0; i < thisList.size(); i++ ) {
          ( ( SmileyListener )thisList.elementAt( i ) ).smileyChanged( thisEvent );
       }
       
    }
    
    //----------------------------------------------------------------------
    // A figyelk regisztrlst biztost metdusok:

    /**
     * Az itt megadott PropertyChangeListener objektumok  propertyChange metdusa
     * lesz meghvva valahnyszor egy kttt tulajdonsg megvltozik.
     * A Javabeans specifikci nem hatrozza meg, hogy milyen sorrendben  
     * hvdjanak meg a figyelk metdusai.
     */      
    public void addPropertyChangeListener( PropertyChangeListener l ) {
       changes.addPropertyChangeListener( l );
    }

    /** 
     * Kiveszi a megadott PropertyChangeListener figyelt a SmileyBean bels 
     * regisztrcis listjbl.
     */      
    public void removePropertyChangeListener( PropertyChangeListener l ) {
       changes.removePropertyChangeListener( l );
    }

    /**
     * Az itt megadott VetoableChangeListeners objektumok vetoableChange metdusa
     * lesz meghvva valahnyszor egy vtzhat tulajdonsg megvltozik.
     * A SmileyBean jelenleg egy ilyen tulajdonsgot tartalmaz: "funPercent".
     * A Javabeans specifikci nem hatrozza meg, hogy milyen sorrendben  
     * hvdjanak meg a figyelk metdusai.
     */      
    public void addVetoableChangeListener( VetoableChangeListener l ) {
       vetos.addVetoableChangeListener( l );
    }

    /** 
     * Kiveszi a megadott VetoableChangeListener figyelt a SmileyBean bels 
     * regisztrcis listjbl.
     */      
    public void removeVetoableChangeListener( VetoableChangeListener l ) {
       vetos.removeVetoableChangeListener( l );
    }

    //----------------------------------------------------------------------
    // Privt adatmezk:

    private PropertyChangeSupport changes = new PropertyChangeSupport( this );
    private VetoableChangeSupport vetos = new VetoableChangeSupport( this );
    
    // A SmileyListener figyelk
    private Vector smileyListeners = new Vector();

    private Color drawColor = Color.black;
    private Color faceColor = Color.yellow;

    private int ourFunPercent = 50;
    private int minChangeToNotify = 10;
    private static final int MAX_SMILEY_ANGLE = 150;
   
    private String colorScheme = COL_DEFAULT;
    
    public static final String COL_DEFAULT = "default";
    public static final String COL_GRAY = "gray";
    public static final String COL_ATROCIOUS = "atrocious";
    public static final String COL_HIDEOUS = "hideous";
    
}

