package javabook.smiley;

import java.awt.*;
import java.beans.*;

/**
 * Egy egyszer bean kttt tulajdonsgokkal s egy vtzhatval.
 * A vtzhat tulajdonsg a "funPercent".  VetoablePropertyChange 
 * figyelk elutasthatjk a tulajdonsg javasolt vltozst egy
 * PropertyVetoException kivtel eldobsval.
 * 
 */
public class SmileyBean2 extends java.awt.Component 
                        implements PropertyChangeListener {

    /**
     * Konstruktor
     */
    public SmileyBean2() {
    }

    public void paint(Graphics g) {
       g.setColor( bgColor );
       g.fillArc( 5, 5, 40, 40, 0, 360 );
       g.setColor( ourColor );
       g.drawArc( 5, 5, 40, 40, 0, 360 );
       g.fillArc( 15, 18, 5, 5, 0, 360 );
       g.fillArc( 30, 18, 5, 5, 0, 360 );
       int a = ( int )( 0.5 + ( ourFunPercent / 100.0 ) * MAX_SMILEY_ANGLE );
       if ( a < 4 ) { a = 0; } // Hogy az v ne legyen kisebb, mint egy pixel
       g.drawArc( 15, 15, 20, 20, -( 180 - a ) / 2, -a );
    }

    /**
     * A propertyChangeEvent kezelje, az esemny alapjn belltja 
     * a funPercent j rtkt
     */
    public void propertyChange( PropertyChangeEvent event ) {
       Integer newValue = ( Integer )event.getNewValue();
       try {
          setFunPercent( newValue.intValue() );
       } catch ( PropertyVetoException e ) {
       }
    }
    
    public Dimension getPreferredSize() {
       return new Dimension(50,50);
    }

    /** 
     * Visszatr rajzolat sznvel
     */
    public synchronized Color getColor() {
        return ourColor;
    }

    /** 
     * Belltja a rajzolat sznt. Ez egy kttt tulajdonsg
     */
    public void setColor( Color newColor ) {
       Color oldColor = ourColor;
       ourColor = newColor;
       changes.firePropertyChange( "color", oldColor, newColor );
       repaint();
    }

    /** 
     * Visszatr az arc sznvel
     */
    public synchronized Color getBgColor() {
        return bgColor;
    }

    /** 
     * Belltja az arc sznt. Ez egy kttt tulajdonsg
     */
    public void setBgColor( Color newColor ) {
       Color oldColor = bgColor;
       bgColor = newColor;
       changes.firePropertyChange( "bgColor", oldColor, newColor );
       repaint();
    }

    /** 
     * Visszaadja az aktulis vidmsg fokozatot szzalkban
     */
    public synchronized int getFunPercent() {
        return ourFunPercent;
    }

    /**
     * Belltja a vidmsg fokozatot szzalkban, hacsak nem dob el egy 
     * regisztrlt VetoableChangeListener egy PropertyVetoException kivtelt.
     * Ez egy vtozhat kttt tulajdonsg.
     */
    public void setFunPercent( int newFunPercent )
                            throws PropertyVetoException {
       if ( newFunPercent <= 100 && newFunPercent >= 0 ) {
         int oldFunPercent = ourFunPercent;

         // Elszr rtestjk a vtzkat a vltozsrl. Ha brmelyik kivtelt dob
         // nem kapjuk el, hanem a fggvny hvjnak delegljuk tovbb.
         vetos.fireVetoableChange("funPercent", 
                    new Integer( oldFunPercent ),
                    new Integer( newFunPercent ) );
         // Ha senki nem vtzott, menjnk tovbb s vltoztassuk meg az rtket
         ourFunPercent = newFunPercent;
         repaint();
         changes.firePropertyChange("funPercent", 
                      new Integer( oldFunPercent ),
                      new Integer( newFunPercent ) );
         
       }
    }

    //----------------------------------------------------------------------
    // A figyelk regisztrlst biztost metdusok:

    /**
     * Az itt megadott PropertyChangeListener objektumok  propertyChange metdusa
     * lesz meghvva valahnyszor egy kttt tulajdonsg megvltozik.
     * A Javabeans specifikci nem hatrozza meg, hogy milyen sorrendben  
     * hvdjanak meg a figyelk metdusai.
     */      
    public void addPropertyChangeListener( PropertyChangeListener l ) {
       changes.addPropertyChangeListener( l );
    }

    /** 
     * Kiveszi a megadott PropertyChangeListener figyelt a SmileyBean bels 
     * regisztrcis listjbl.
     */      
    public void removePropertyChangeListener( PropertyChangeListener l ) {
       changes.removePropertyChangeListener( l );
    }

    /**
     * Az itt megadott VetoableChangeListeners objektumok vetoableChange metdusa
     * lesz meghvva valahnyszor egy vtzhat tulajdonsg megvltozik.
     * A SmileyBean jelenleg egy ilyen tulajdonsgot tartalmaz: "funPercent".
     * A Javabeans specifikci nem hatrozza meg, hogy milyen sorrendben  
     * hvdjanak meg a figyelk metdusai.
     */      
    public void addVetoableChangeListener( VetoableChangeListener l ) {
       vetos.addVetoableChangeListener( l );
    }

    /** 
     * Kiveszi a megadott VetoableChangeListener figyelt a SmileyBean bels 
     * regisztrcis listjbl.
     */      
    public void removeVetoableChangeListener( VetoableChangeListener l ) {
       vetos.removeVetoableChangeListener( l );
    }

    //----------------------------------------------------------------------
    // Privt adatmezk:

    private PropertyChangeSupport changes = new PropertyChangeSupport( this );
    private VetoableChangeSupport vetos = new VetoableChangeSupport( this );

    private Color ourColor = Color.black;
    private Color bgColor = Color.yellow;
    private int ourFunPercent = 50;
    private static final int MAX_SMILEY_ANGLE = 150;
    
}

