/***** Java Chat Program - Szerveroldal implementcija *****/

import java.rmi.Naming;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.util.*; 

public class RMIChatServer extends UnicastRemoteObject 
                  implements RMIChatServerIf {

  Hashtable resztvevok; /* rsztvevk beceneve s "kliensoldali" objektumuk */

  public RMIChatServer() throws java.rmi.RemoteException {
    super(); /* Ez exportlja a tvoli objektumot */
    try {
      resztvevok=new Hashtable();
    } catch (Exception e) {
      System.out.println("ChatServer objektum nem hozhat ltre.");
    }
  }

  /***
     * Az sszes rsztvevnek eljuttat egy (nem res) zenetet
     ***/
  private Hashtable resztvevokErtesitese(String nev, String uzenet) 
  {
    Hashtable kiesettek;

    kiesettek=new Hashtable();
    synchronized (resztvevok) {
      Enumeration rvEnum=resztvevok.keys();
      while (rvEnum.hasMoreElements()) {   /* Az sszes rsztvevnek ... */
        String rnev=(String) rvEnum.nextElement();
        RMIChatClientIf r=(RMIChatClientIf) resztvevok.get(rnev);
        try {
          if (!uzenet.equals(""))          /* Ha a szveg nem res ... */
          {
            r.uzenetFogadas(nev,uzenet);   /* akkor elkldjk a kliensnek */
          }
        } catch (java.rmi.RemoteException re)
        {
          /* rnev rsztvev nem elrhet - felveszem mint kiesettet */
          kiesettek.put(rnev,resztvevok.get(rnev));
          resztvevok.remove(rnev);
        }
      }
    }
    return kiesettek; /* Visszaadom a kiesettek nvsort */
  }

  /***
     * Az sszes rsztvevnek eljuttat egy "XY belpett/kilpett" zenetet
     * Itt az zenet kirsa mellett a kliensoldali trsak listjnak
     * aktualizlsa fontos, ebben klnbzik a norml kir metdustl.
     ***/
  private Hashtable kibelepesErtesites(String nev, boolean ki_e) 
  {
    Hashtable kiesettek;

    kiesettek=new Hashtable();
    synchronized (resztvevok) {
      Enumeration rvEnum=resztvevok.keys();
      while (rvEnum.hasMoreElements()) {
        String rnev=(String) rvEnum.nextElement();
        RMIChatClientIf r=(RMIChatClientIf) resztvevok.get(rnev);
        try {
          if (ki_e) {
            r.uzenetFogadas("",nev+" kilpett.");
            r.tarsKilepes(nev);
          } else {
            r.uzenetFogadas("",nev+" belpett a szobba.");
            r.tarsBelepes(nev);
          }
        } catch (java.rmi.RemoteException re)
        {
          /* rnev rsztvev nem elrhet - felveszem mint kiesettet */
          kiesettek.put(rnev,resztvevok.get(rnev));
          resztvevok.remove(rnev);
        }
      }
    }
    return kiesettek;
  }
  
  /***
     * Az sszes rsztvevnek eljuttat egy "XY kilpett" zenetet
     * Itt az zenet kirsa mellett a kliensoldali trsak listjnak
     * aktualizlsa fontos, ebben klnbzik a norml kir metdustl.
     ***/
  private Hashtable kilepesErtesites(String nev) 
  {
    return kibelepesErtesites(nev,true);
  }

  /***
     * Az sszes rsztvevnek eljuttat egy "XY belpett" zenetet
     * Itt az zenet kirsa mellett a kliensoldali trsak listjnak
     * aktualizlsa fontos, ebben klnbzik a norml kir metdustl.
     ***/
  private Hashtable belepesErtesites(String nev) 
  {
    return kibelepesErtesites(nev,false);
  }

  /***
     * Kitrli a rsztvevk nvsorbl a paramterben megadott
     * (elrhetetlennek vlt) klienseket.
     * A kiesettnek vltek mr ki vannak vve a rsztvevk kzl,
     * gy az rtests csak az aktv klienseknek lesz elkldve.
     * Ha kzben - az rtestsek sorn - valaki kiesne ... akkor jbl
     * tesznk egy krt ... (vgessg oka: rekurzv meghvs sorn a
     * rsztvevk szma cskken - vgtelen ciklus e rekurzibl nem lehet).
     ***/
  public void RIP(Hashtable kiesettek) 
  {
    synchronized (kiesettek) {
      Enumeration rvEnum=kiesettek.keys();
      while (rvEnum.hasMoreElements()) {            /* Az sszes kiesettrl */
        String rnev=(String) rvEnum.nextElement();
        Hashtable kiesettek2=                       /* zenet az lknek    */
                   resztvevokErtesitese("",rnev+" kapcsolata bontott!");
        RIP(kiesettek2); /* jabb kiesettek ... bke poraikra ... */
      }
    }
  }

  /***
     * A msodik paramterben megadott nev csetel kirgta az els
     * paramterben megadott nev trst.
     * Ha ltezik a kirgott nev csetes, kitesszk a csetbl s 
     * rtestjk a tbbieket.
     * Megj.: jelenleg mindenki rgdoshat - ide lehetne berni pldul
     * olyan ellenrzst, hogy csak bizonyos nev csetesek rghatnak ki
     * msokat. Mivel ez nem bonyolult algoritmus (milyen nevet adtunk
     * az "if" utastsnak :-) ), s nem csetspecifikus, ezrt itt most
     * ilyent nem csinlunk.
     ***/
  private void kirug(String nev, String ki_rugta_ki)
  {
    boolean letezonev=false;
    synchronized (resztvevok) 
    {
      if (letezonev=resztvevok.containsKey(nev))
      {
         RMIChatClientIf r=(RMIChatClientIf) resztvevok.remove(nev);
         /* A kirgottat kiszedtk a rsztvevk kzl, s rtestjk ... */
         try {
           r.uzenetFogadas("","Kirgtak a csetbl,biztos j okuk volt r.");
           r.tarsKilepes(nev);
           r.kirugtak();
         } catch (java.rmi.RemoteException re)
         {
          // kirgott rsztvev nem elrhet - nem rdekel, gyis kitettk :-)
         }
      }
    }
    if (letezonev)
    {
      /* Ha kirgtunk valakit, rtestjk a tbbieket */
      Hashtable kiesettek = 
         resztvevokErtesitese("",
                    nev+" kilepett a szobabol, mert "+ki_rugta_ki+" kirugta.");
      RIP(kiesettek);
    }
  }

  public int uzenetFogadas(String nev, String szoveg) 
              throws java.rmi.RemoteException 
  {
    char elsojelhavan=' ';

    if (szoveg.length() > 0)
    {
      elsojelhavan=szoveg.charAt(0);
    }

    /* Ha az els jel nem egy /,akkor visszarjuk az zenetet a csetelknek */
    if (elsojelhavan != '/')
    {
      Hashtable kiesettek = resztvevokErtesitese(nev,szoveg);
      RIP(kiesettek);
    } else { /* Egy parancsrl van sz */
      if (szoveg.startsWith("/kirug ")) /* kirgs? */
      {
        kirug(szoveg.substring(7),nev); /* ha igen, kitesszk az ldozatot */
      } else System.err.println("Ismeretlen parancs ...");
    }
    return resztvevok.size(); /* Visszaadjuk a rsztvevk szmt */
  }

  public Hashtable nevlistaSzinkronizalas() throws java.rmi.RemoteException
  {
    Hashtable csaknevek=new Hashtable();

    /* sszelltunk egy hashtblt, benne kulcsknt a rsztvevk neveivel */
    synchronized (resztvevok) {
      Enumeration rvEnum=resztvevok.keys();
      while (rvEnum.hasMoreElements()) {
        String rnev=(String) rvEnum.nextElement();
        csaknevek.put(rnev,"mindegymi");
      }
    }
    return csaknevek; /* Visszaadjuk a hashtblt a kliensnek */
  }

  public void tarsBelepes(String nev, RMIChatClientIf Kli) 
              throws java.rmi.RemoteException
  {
    // System.out.println("Belepett: "+nev); // Ez most nem kell ...
    // feltltjk az uj bejelentkez nev-listajat
    synchronized (resztvevok) {  /* Felvesszk a belpt a rsztvevk kz  */
      resztvevok.put(nev,Kli);   /* IP cm alapjni tilts esetn itt lehet-*/
    }                            /* ne visszautastani a krnikus rosszakat */

    Hashtable kiesettek=new Hashtable(); /* Ha kzben kiesne valaki ... */

    /* j belp nvlistja */
    synchronized (resztvevok) {
      Enumeration rvEnum=resztvevok.keys();
      while (rvEnum.hasMoreElements()) {
        String rnev=(String) rvEnum.nextElement();
        try {
          if (!rnev.equals(nev)) {
            Kli.tarsBelepes(rnev); /* Nvlista aktualizlsa rsztvevknl */
          }
        } catch (java.rmi.RemoteException re)
        {
          /* rnev rsztvev nem elrhet? Pedig most akart volna belpni */
          kiesettek.put(rnev,resztvevok.get(rnev));
          resztvevok.remove(rnev);
          System.err.println("tarsBelepes: kliens visszahvs sikertelen");
        }
      }
    } 
    RIP(kiesettek);
    kiesettek=belepesErtesites(nev);
    RIP(kiesettek);
  }

  public void tarsKilepes(String nev) throws java.rmi.RemoteException
  {
    // System.out.println("Kilepett: "+nev);  // Ez most nem kell ...
    synchronized (resztvevok) {
      RMIChatClientIf Kli = (RMIChatClientIf) resztvevok.remove(nev);
    }
    /* Tbbiek rtestse */
    Hashtable kiesettek = kilepesErtesites(nev);
    RIP(kiesettek);
  }

  /***
     * A szerver fprogramja 
     * Ltrehozzuk a szerverobjektumot, bejegyezzk a registrybe.
     * A tbbi mr csak a kliensek ... azaz a csetelk feladata ...
     ***/
  public static void main(String args[]) {
    System.setSecurityManager(new RMISecurityManager());
    String hn="localhost";
    if (args.length > 0) hn=args[0];
    try {
      RMIChatServer n = new RMIChatServer();
      Naming.rebind("rmi://"+hn+"/DefaultChatSzoba",n);
      System.out.println("ChatServer fut ...");
    } catch (Exception e) {
      System.out.println("ChatSzerver nem indthat ...");
      System.out.println("Hiba oka:"+e.getMessage());
    }
  }
}
