/**
 * AranyLayout osztly
 *   A felhasznlhat terletarny megadsval vezrelhet elrendezsi stratgia
 */
public class AranyLayout implements java.awt.LayoutManager2 {
  /**
   * Komponens hozzadsa nvvel
   */
  public void addLayoutComponent(String pNv, java.awt.Component pKomponens) {
    addLayoutComponent( pKomponens, pNv );
  }
  // A komponensekhez rendelt elrendezsi jellemzket trolja
  private java.util.HashMap jellemzk = new java.util.HashMap(7);
  // elrendezsi jellemz lekrdezse
  private AranyConstraints getAranyConstraints(java.awt.Component pKomponens) {
    AranyConstraints jellemz = (AranyConstraints)jellemzk.get( pKomponens );
    if (jellemz == null) throw new IllegalArgumentException(
       "AranyConstraints nincs megadva: "+pKomponens);
    return jellemz;
  }
  /**
   * Komponens hozzadsa elrendezsi jellemzvel
   */
  public void addLayoutComponent(java.awt.Component pKomponens,
                                 Object pJellemz) {
    if (!(pJellemz instanceof AranyConstraints)) //csak a megfelel elrendezsi
       throw new IllegalArgumentException(                //jellemz megengedett
                 "AranyConstraints nincs megadva: "+pKomponens);
    jellemzk.put( pKomponens, pJellemz );
  }
  /**
   * Komponens trlse az elrendezsi stratgibl
   */
  public void removeLayoutComponent(java.awt.Component pKomponens) {
    jellemzk.remove( pKomponens );
  }
  /**
   * Vzszintes igaztsi pont mindig kzpen lesz
   */
  public float getLayoutAlignmentX(java.awt.Container pKontner) {
    return java.awt.Component.CENTER_ALIGNMENT;
  }
  /**
   * Fggleges igaztsi pont mindig kzpen lesz
   */
  public float getLayoutAlignmentY(java.awt.Container pKontner) {
    return java.awt.Component.CENTER_ALIGNMENT;
  }
  /*****************************************************************************
  */
  private interface MeretSzamolo {         //segdinterfsz a mretkiszmolshoz
    public java.awt.Dimension mret(java.awt.Component pKomponens);
  }
  private static final MeretSzamolo MINIMUM = new MeretSzamolo() {   //minimlis
    public java.awt.Dimension mret(java.awt.Component pKomponens) {  //mrethez
      return pKomponens.getMinimumSize();
    }
  };
  private static final MeretSzamolo PREFERRED = new MeretSzamolo() { //optimlis
    public java.awt.Dimension mret(java.awt.Component pKomponens) {  //mrethez
      return pKomponens.getPreferredSize();
    }
  };
  private static final MeretSzamolo MAXIMUM = new MeretSzamolo() {   //maximlis
    public java.awt.Dimension mret(java.awt.Component pKomponens) {  //mrethez
      return pKomponens.getMaximumSize();
    }
  };
  /**
   * Minimlis mret kiszmtsa
   */
  public java.awt.Dimension minimumLayoutSize(java.awt.Container pKontner) {
    return mret( pKontner, MINIMUM );
  }
  /**
   * Optimlis mret kiszmtsa
   */
  public java.awt.Dimension preferredLayoutSize(java.awt.Container pKontner) {
    return mret( pKontner, PREFERRED );
  }
  /**
   * Maximlis mret kiszmtsa
   */
  public java.awt.Dimension maximumLayoutSize(java.awt.Container pKontner) {
    return mret( pKontner, MAXIMUM );
  }
  private java.awt.Dimension mret(java.awt.Container pKontner, //kontnermret
                                   MeretSzamolo pMeretSzamolo) {   //kiszmtsa
    int szlessg = 0, magassg = 0;           //kiszmtott szlessg, magassg
    java.awt.Component komponens = null;                    //aktulis komponens
    AranyConstraints jellemz = null;            //aktulis elrendezsi jellemz
    float xarny = 0, yarny = 0;        //aktulis vzszintes, fggleges arny
    java.awt.Dimension mret = null;                 //aktulis komponens mrete
    java.awt.Component[] komponensek = pKontner.getComponents();
    if (komponensek!=null) for (int i=0; i<komponensek.length; i++) {
       komponens = komponensek[i];
       if (!komponens.isVisible()) continue;      //csak a lthat komponenseket
       jellemz = getAranyConstraints( komponens );
       xarny = jellemz.getXarny();
       yarny = jellemz.getYarny();
       mret = pMeretSzamolo.mret( komponens );
       if (xarny == AranyConstraints.EREDETIMRET)  //eredeti mretben befrjen
          szlessg = Math.max(szlessg, mret.width);
       else if (xarny>0)                         //arnyosan mretezve befrjen
          szlessg = Math.max(szlessg, Math.round(1/xarny * mret.width));
       if (yarny == AranyConstraints.EREDETIMRET)  //eredeti mretben befrjen
          magassg = Math.max(magassg, mret.height);
       else if (yarny>0)                         //arnyosan mretezve befrjen
          magassg = Math.max(magassg, Math.round(1/yarny * mret.height));
    }
    java.awt.Insets keretvastagsg = pKontner.getInsets();     //keretvastagsg
    if (keretvastagsg != null) {                            //figyelembe vtele
       if (szlessg<Integer.MAX_VALUE)
          szlessg += keretvastagsg.left + keretvastagsg.right;
       if (magassg<Integer.MAX_VALUE)
          magassg += keretvastagsg.top + keretvastagsg.bottom;
    }
    return new java.awt.Dimension(szlessg, magassg);
  }
  /**
   * Elrendezsi stratgia megvalstsa
   */
  public void layoutContainer(java.awt.Container pKontner) {
    int sszszlessg = pKontner.getWidth();              //kontner szlessge
    int sszmagassg = pKontner.getHeight();               //kontner magassga
    int startx = 0, starty = 0;                   //kontner els lthat pontja
    java.awt.Insets keretvastagsg = pKontner.getInsets();     //keretvastagsg
    if (keretvastagsg != null) {                            //figyelembe vtele
       sszszlessg -= keretvastagsg.left + keretvastagsg.right;
       sszmagassg -= keretvastagsg.top + keretvastagsg.bottom;
       startx = keretvastagsg.left;
       starty = keretvastagsg.top;
    }
    java.awt.Component[] komponensek = pKontner.getComponents();
    if (komponensek == null) return;                           //res a kontner
    int kezdindex = 0;                              //sorkezd komponens indexe
    while (kezdindex < komponensek.length &&                //ennek kiszmtsa
          !komponensek[kezdindex].isVisible()) kezdindex++;
    java.awt.ComponentOrientation irny =    //komponensek sorbarendezsi irnya
        pKontner.getComponentOrientation();
    if (irny==null) irny = java.awt.ComponentOrientation.getOrientation(
       java.util.Locale.getDefault());    //alaprtelmezett sorbarendezsi irny
    if (irny.isHorizontal()) sorokbaRendez(komponensek, kezdindex,
       sszszlessg, sszmagassg, startx, starty, irny.isLeftToRight());
    else oszlopokbaRendez(komponensek, kezdindex,
       sszszlessg, sszmagassg, startx, starty, irny.isLeftToRight());
  }
  //komponensek sorokba rendezse
  private void sorokbaRendez(java.awt.Component[] pKomponensek, int pKezdIndex,
              int psszSzlessg, int psszMagassg, int pStartX, int pStartY,
              boolean pBalrlJobbra_e ) {
    java.awt.Component komponens = null;                    //aktulis komponens
    AranyConstraints jellemz = null;            //aktulis elrendezsi jellemz
    float xarny = 0, yarny = 0;        //aktulis vzszintes, fggleges arny
    int kezdx = pStartX+(pBalrlJobbra_e ? 0 : psszSzlessg); //kezd pozci
    int x=kezdx,y=pStartY,szlessg=0,magassg=0; //aktulis komponens helyzete
    java.awt.Dimension optimret = null;   //aktulis komponens optimlis mrete
    int sormagassg1 = 0;                               //sormagassg fels fele
    int sormagassg2 = 0;                                //sormagassg als fele
    int maradkszlessg = psszSzlessg;         //a sorban mg megmaradt hely
    int igaztsipont = 0;          //komponens figyelembe vett igaztsi pontja

    for (int i = pKezdIndex; i < pKomponensek.length; i++) {
      komponens = pKomponensek[i];
      if (!komponens.isVisible()) continue;       //csak a lthat komponenseket
      jellemz = getAranyConstraints( komponens );
      xarny = jellemz.getXarny();
      yarny = jellemz.getYarny();
      if (xarny == AranyConstraints.EREDETIMRET) {         //eredeti szlessg
         optimret = komponens.getPreferredSize();
         szlessg = optimret.width;
      } else szlessg = Math.round( psszSzlessg*xarny );//arnyos szlessg
      if (yarny == AranyConstraints.EREDETIMRET) {          //eredeti magassg
         if (xarny != AranyConstraints.EREDETIMRET)  //ha mg nem krdeztk le
            optimret = komponens.getPreferredSize();
         magassg = optimret.height;
      } else magassg = Math.round( psszMagassg * yarny ); //arnyos magassg
      igaztsipont = Math.round( magassg * komponens.getAlignmentY() );
      if (maradkszlessg < szlessg &&  i > pKezdIndex) {     //sor lezrsa
         for (int ii=pKezdIndex; ii<i; ii++) {          //komponensek igaztsa
             komponens = pKomponensek[ii];                       //a soron bell
             if (komponens.isVisible())
                komponens.setLocation(komponens.getX(), y + sormagassg1 -
                   Math.round(komponens.getHeight()*komponens.getAlignmentY()));
         }
         komponens = pKomponensek[i];
         pKezdIndex = i;                                       //j sor kezdse
         x = kezdx;    
         y += sormagassg1 + sormagassg2;             
         sormagassg1 = igaztsipont;
         sormagassg2 = magassg - igaztsipont;
         maradkszlessg = psszSzlessg;
      } else {
        sormagassg1 = Math.max( sormagassg1, igaztsipont );    //sormagassg
        sormagassg2 = Math.max( sormagassg2, magassg - igaztsipont );
      }
      if (pBalrlJobbra_e) {                             //komponens elhelyezse
         komponens.setBounds(x, y, szlessg, magassg);
         x += szlessg;
      } else {
        x -= szlessg;
        komponens.setBounds(x, y, szlessg, magassg);
      }
      maradkszlessg -= szlessg;
    }
    for (int i=pKezdIndex; i<pKomponensek.length; i++) { //utols sor igaztsa
      komponens = pKomponensek[i];
      if (komponens.isVisible())
         komponens.setLocation(komponens.getX(), y + sormagassg1 -
                   Math.round(komponens.getHeight()*komponens.getAlignmentY()));
    }
  }
  //komponensek oszlopokba rendezse
  private void oszlopokbaRendez( java.awt.Component[] pKomponensek, int 
   pKezdIndex, int psszSzlessg, int psszMagassg, int pStartX, int pStartY,
   boolean pBalrlJobbra_e ){
    java.awt.Component komponens = null;                    //aktulis komponens
    AranyConstraints jellemz = null;            //aktulis elrendezsi jellemz
    float xarny = 0, yarny = 0;        //aktulis vzszintes, fggleges arny
    int x = pStartX + (pBalrlJobbra_e ? 0 : psszSzlessg); //aktulis helyzet
    int y = pStartY, szlessg = 0, magassg = 0;
    java.awt.Dimension optimret = null;   //aktulis komponens optimlis mrete
    int oszlopszlessg1 = 0;                         //oszlopszlessg bal fele
    int oszlopszlessg2 = 0;                        //oszlopszlessg jobb fele
    int maradkmagassg = psszMagassg;       //az oszlopban mg megmaradt hely
    int igaztsipont = 0;          //komponens figyelembe vett igaztsi pontja

    for (int i = pKezdIndex; i < pKomponensek.length; i++) {
      komponens = pKomponensek[i];
      if (!komponens.isVisible()) continue;       //csak a lthat komponenseket
      jellemz = getAranyConstraints( komponens );
      xarny = jellemz.getXarny();
      yarny = jellemz.getYarny();
      if (xarny == AranyConstraints.EREDETIMRET) {         //eredeti szlessg
         optimret = komponens.getPreferredSize();
         szlessg = optimret.width;
      } else szlessg = Math.round( psszSzlessg*xarny );//arnyos szlessg
      if (yarny == AranyConstraints.EREDETIMRET) {          //eredeti magassg
         if (xarny != AranyConstraints.EREDETIMRET)  //ha mg nem krdeztk le
            optimret = komponens.getPreferredSize();
         magassg = optimret.height;
      } else magassg = Math.round( psszMagassg * yarny ); //arnyos magassg
      igaztsipont = Math.round( szlessg * komponens.getAlignmentX() );
      if (maradkmagassg < magassg &&  i > pKezdIndex) {    //oszlop lezrsa
         for (int ii=pKezdIndex; ii<i; ii++) {          //komponensek igaztsa
             komponens = pKomponensek[ii];                   //az oszlopon bell
             if (komponens.isVisible()) komponens.setLocation( x +
                (pBalrlJobbra_e ?  oszlopszlessg1 : -oszlopszlessg2) -
                 Math.round(komponens.getWidth() * komponens.getAlignmentX()),
                 komponens.getY());
         }
         komponens = pKomponensek[i];
         pKezdIndex = i;                                    //j oszlop kezdse
         if (pBalrlJobbra_e) x+=oszlopszlessg1 + oszlopszlessg2;
         else x-=oszlopszlessg1 + oszlopszlessg2;
         y = pStartY;             
         oszlopszlessg1 = igaztsipont;
         oszlopszlessg2 = szlessg - igaztsipont;
         maradkmagassg = psszMagassg;
      } else {                                                 //oszlopszlessg
        oszlopszlessg1 = Math.max( oszlopszlessg1, igaztsipont );
        oszlopszlessg2 = Math.max(oszlopszlessg2, szlessg-igaztsipont);
      }
      if (pBalrlJobbra_e)                               //komponens elhelyezse
         komponens.setBounds(x, y, szlessg, magassg);
      else komponens.setBounds(x-szlessg, y, szlessg, magassg);
      y += magassg;
      maradkmagassg -= magassg;
    }
    for (int i=pKezdIndex; i<pKomponensek.length; i++) {        //utols oszlop
      komponens = pKomponensek[i];
      if (komponens.isVisible()) komponens.setLocation( x +
         (pBalrlJobbra_e ?  oszlopszlessg1 : -oszlopszlessg2) -
          Math.round(komponens.getWidth() * komponens.getAlignmentX()),
          komponens.getY());
    }
  }
  /**
   * Megjelents rvnytelentse
   */
  public void invalidateLayout(java.awt.Container pKontner) {}
}
