package jc;

import javacard.framework.*;
import javacardx.framework.*;

public class Phone extends Applet{

	final static byte	SELECT	= (byte)0xA4;	//INS value for ISO 7816-4 SELECT command
    final static short  CREDIT_FID  = (short) 0x9103; //a creditFile FID-je
    final static short	CREDIT_LENGTH =  2;			//az egysgek szmt 2 bjton troljuk
    final static byte   ISO_CLA = (byte)0x00;	//az ISO 7816-4 fjlkezel utastsok CLA bjtja
    final static byte	PHONE_CLA	= (byte)0xAB;	//a Phone applet CLA byte-ja
    final static byte 	SET_CREDIT	= (byte)0x10;	//a SetCredit utasts INS bjtja
    final static byte 	DEC_CREDIT	= (byte)0x20;	//a DecCredit utasts INS bjtja
    final static byte 	START	=	0;			//nulls offszet buffer-mveletekhez
    final static short	NO_MORE_CREDITS 
    	= (short)0x9101;		//SW bjtok a "nincs tbb egysg" esemny jelzsre

    FileSystem      fs;
	TransparentFile	creditFile;
  	boolean justSelected;
  	
    private Phone(){
 		//itt kell lefoglalni az sszes szksges memrit
    	//gy nem fogyhat el a memria futs kzben

 		//az ISO fjlrendszer ltrehozsa
        fs = new FileSystem((byte)1);
        creditFile = new TransparentFile(CREDIT_FID,CREDIT_LENGTH);
        fs.addChildFile(creditFile);
   		creditFile.setSecurity(File.ACCESS_READ,File.ALLOW_ANY); //mindenki olvashatja
		creditFile.setSecurity(File.ACCESS_WRITE,File.ALLOW_NONE); //senki sem rhatja
	}//Phone()

    public static void install( APDU apdu ){
    	Phone phone = new Phone();
    	phone.register();
    }
    public boolean select(){
    	justSelected = true;
    	fs.reset();
        return true;
    }

    public void process(APDU apdu){
        byte[] buffer = apdu.getBuffer();
        if (buffer[ISO.OFFSET_CLA] == PHONE_CLA) {
	        switch (buffer[ISO.OFFSET_INS]){
	        	case SET_CREDIT:
					processSetCredit(apdu); break;
				case DEC_CREDIT:
					processDecCredit(apdu); break;
				default:
					ISOException.throwIt(ISO.SW_INS_NOT_SUPPORTED);
			}//switch
        }//if
        else if (buffer[ISO.OFFSET_CLA] == ISO_CLA) {
        	if ((buffer[ISO.OFFSET_INS] == SELECT) & justSelected) {
        	// most vlasztottk ki az appletnket
				processSelected();
			}else
		    //Minden ms ISO-kompatibilis utastst a FileSystem osztly kezel le
			if (!fs.process(apdu)) ISOException.throwIt(ISO.SW_INS_NOT_SUPPORTED);
		}
		else ISOException.throwIt(ISO.SW_CLA_NOT_SUPPORTED);
	}

	void processSelected(){
		justSelected = false;
	}


	void processSetCredit(APDU apdu){
		byte[] buffer = apdu.getBuffer();
		byte[] credit = creditFile.getData();
		short cr;
		if (buffer[ISO.OFFSET_P1] != 0)
			ISOException.throwIt(ISO.SW_INCORRECT_P1P2);
		if (buffer[ISO.OFFSET_P2] != 0)
			ISOException.throwIt(ISO.SW_INCORRECT_P1P2);
		if (buffer[ISO.OFFSET_LC] != CREDIT_LENGTH)
			ISOException.throwIt(ISO.SW_WRONG_LENGTH);
		apdu.setIncomingAndReceive();    //beolvassuk az adatokat
		if (tdk.ID.p_masterPIN.isValidated()){ //mr megadtk a Master PIN-t
			Util.arrayCopy(buffer,ISO.OFFSET_CDATA,credit,START,CREDIT_LENGTH);
			cr=Util.getShort(credit,START);
			Util.setShort(buffer,START,cr);
			apdu.setOutgoingAndSend((short)0, (byte)2);
			return;
		}
		else ISOException.throwIt(ISO.SW_CONDITIONS_NOT_SATISFIED);
	}

	void processDecCredit(APDU apdu){
		byte[] buffer = apdu.getBuffer();
		byte[] credit = creditFile.getData();
		short cr;
		if (buffer[ISO.OFFSET_P1] != 0)
			ISOException.throwIt(ISO.SW_INCORRECT_P1P2);
		if (buffer[ISO.OFFSET_P2] != 0)
			ISOException.throwIt(ISO.SW_INCORRECT_P1P2);
		if (buffer[ISO.OFFSET_LC] != 0)
			ISOException.throwIt(ISO.SW_WRONG_LENGTH);
		apdu.setIncomingAndReceive();    //beolvassuk az adatokat
		if (tdk.ID.p_userPIN.isValidated()){
			cr=Util.getShort(credit,START);
			if (cr==0) //mr nincs tbb egysg
				ISOException.throwIt(NO_MORE_CREDITS);
			else { //van mg egysg
				cr--;
				Util.setShort(credit,START,cr);
				Util.setShort(buffer,START,cr);
				apdu.setOutgoingAndSend((short)0, (byte)2);
				return;
			}
		}
		else ISOException.throwIt(ISO.SW_CONDITIONS_NOT_SATISFIED);
	}

}//Phone