import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.sql.*;
import java.applet.Applet;

/**
 * JDBCTest osztly - ltalnos JDBC konzolprogram
 */
public class JDBCTest extends JApplet implements ActionListener {
  JButton registerButton=new JButton("Betlt");                 // Felletelemek
  JTextField driver=new JTextField();
  JButton connectButton=new JButton("Megnyit");
  JTextField url=new JTextField();
  JTextField userid=new JTextField(10);
  JPasswordField password=new JPasswordField(10);
  JTextArea sql=new JTextArea(5,80);
  JTextArea result=new JTextArea();
  JButton clearButton=new JButton("Trl");
  JButton execButton=new JButton("Vgrehajt");
  JButton listButton=new JButton("Elrhet tblk");

  Connection con;                                         // Adatbzis kapcsolat
  DatabaseMetaData meta;                                   // s annak jellemzi
  
  public JDBCTest() {
    super();
    JPanel panel=new JPanel(new GridLayout(3, 1));         // Fellet felptse
    JPanel driverpanel=new JPanel(new BorderLayout());
    driverpanel.add("West", new JLabel("Meghajtprogram: "));
    driverpanel.add("Center", driver);
    registerButton.addActionListener(this);
    driverpanel.add("East", registerButton);
    panel.add(driverpanel);
    JPanel urlpanel=new JPanel(new BorderLayout());
    urlpanel.add("West", new JLabel("Adatbzis URL: "));
    urlpanel.add("Center", url);
    urlpanel.add("East", connectButton);
    connectButton.addActionListener(this);
    panel.add(urlpanel);
    JPanel passpanel=new JPanel();
    passpanel.add(new JLabel("Felhasznl:"));
    passpanel.add(userid);
    passpanel.add(new JLabel("Jelsz:"));
    password.setEchoChar('*');
    passpanel.add(password);
    panel.add(passpanel);
    getContentPane().add("North", panel);
    JPanel textPanel=new JPanel(new BorderLayout());
    JPanel sqlPanel=new JPanel(new BorderLayout());
    sqlPanel.add("North", new JLabel("Sql:"));
    sqlPanel.add("Center", new JScrollPane(sql));
    textPanel.add("North", sqlPanel);
    JPanel resultPanel=new JPanel(new BorderLayout());
    resultPanel.add("North", new JLabel("Eredmny:"));
    result.setEditable(false);
    result.setFont(new Font("Monospaced", Font.PLAIN, 10));
    resultPanel.add("Center", new JScrollPane(result));
    textPanel.add("Center", resultPanel);
    getContentPane().add("Center", textPanel);
    JPanel buttonPanel=new JPanel();
    clearButton.addActionListener(this);
    buttonPanel.add(clearButton);
    execButton.addActionListener(this);
    buttonPanel.add(execButton);
    listButton.addActionListener(this);
    buttonPanel.add(listButton);
    getContentPane().add("South", buttonPanel);
    validate();                            // Lssuk a meghajtprogram zeneteit
    DriverManager.setLogWriter(new java.io.PrintWriter(System.out));
  }
  
  public static void main (String args[]) {            // indts alkalmazsknt
    JDBCTest myJDBCtest=new JDBCTest();
    JFrame frame=new JFrame(myJDBCtest.getClass().getName()+" alkalmazs");
    frame.addWindowListener(
        new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                   System.exit(0);                        //kilps a programbl
            }
        }
    );
    frame.getContentPane().add("Center", myJDBCtest);
    frame.pack();
    frame.show();
  }
  
  private void kiir(String text) {     //szveg megjelentse az eredmnymezben
    result.append(text+"\n");
  }
  
  private void SQLhibu(SQLException e) {    //hibk/figyelmeztetsek kilistzsa
    String s=e instanceof SQLWarning ? "Figyelmeztets" : "Hiba";
    while (e!=null) {
          kiir("SQLState: "+e.getSQLState());
          kiir(s+" szvege: "+e.getMessage());
          kiir(s+" kdja: "+e.getErrorCode());
          if (e instanceof DataTruncation) {         //DataTruncation lekezelse
             DataTruncation dt=(DataTruncation)e;
             String ds=". ";
             ds+=dt.getParameter() ? "paramter " : "oszlop ";
             ds+=dt.getRead() ? "olvas" : "r";
             kiir("Adatcsonkts a(z) "+dt.getIndex()+ds+"sakor: "+
                  dt.getDataSize()+" -> "+dt.getTransferSize());
          }
          e=e instanceof SQLWarning ? ((SQLWarning)e).getNextWarning()    //lnc
                                    : e.getNextException();          // kvetse
       }
  }

  private void hiba(String s, Throwable e) {                    //hiba kijelzse
    kiir("--- HIBA ---");
    kiir(s+"!");
    kiir(e.toString());
    if (e instanceof SQLException) SQLhibu((SQLException)e);
  }

  private boolean figyu(SQLWarning w) {           //figyelmeztetsek kilistzsa
    if (w!=null) {
       kiir("--- FIGYELMEZTETS ---\n"+w);
       kiir(w.toString());
       SQLhibu(w);
       return true;
    }
    return false;
  }
  
  private String formaz(String s, int width) {  //szveg adott hosszra formzsa
    StringBuffer sb;
    if (s==null) sb=new StringBuffer("null");
    else sb=new StringBuffer(s);
    sb.setLength(width);
    while (width>0 && sb.charAt(--width)=='\u0000') sb.setCharAt(width, ' ');
    return sb.toString();
  }
  
  private void tablakiir(ResultSet rs) throws SQLException {     //eredmnytbla
    boolean insert = false;
    boolean update = false;
    boolean delete = false;
    int widths[];                                                 // kilistzsa
    String s="   ";            // Insert Update Delete detektlst jelz oszlop
    kiir("IUD --- Tbla eleje ---");
    ResultSetMetaData rsmd = rs.getMetaData();
    int numCols = rsmd.getColumnCount();
    widths=new int[numCols];
    for (int i=1; i<=numCols; i++) {              //oszlopok fejlceinek kirsa
        s+=" ";
        widths[i-1]=Math.max(rsmd.getColumnDisplaySize(i),
                             rsmd.getColumnLabel(i).length());
        s+=formaz(rsmd.getColumnLabel(i), widths[i-1]);
    }
    kiir(s);
    try {                                                             //JDBC 2.0
        insert = meta.insertsAreDetected(rs.getType());
        update = meta.updatesAreDetected(rs.getType());
        delete = meta.deletesAreDetected(rs.getType());
        rs.setFetchDirection(ResultSet.FETCH_FORWARD);
    } catch (Throwable e) {}
    while (rs.next()) {                            //tbla tartalmnak listzsa
          try {                                                       //JDBC 2.0
              rs.refreshRow();                //a legaktulisabb adatok kellenek         
          } catch (Throwable e) {}
          s="";                             // detektlt vltoztatsok listzsa
          if (insert) s+=(rs.rowInserted() ? "+" : "-");
          else s+="?";
          if (update) s+=(rs.rowUpdated() ? "+" : "-");
          else s+="?";
          if (delete) s+=(rs.rowDeleted() ? "+" : "-");
          else s+="?";
          for (int i=1; i<=numCols; i++) {       //aktulis sor mezinek kirsa
              s+=" ";
              s+=formaz(rs.getString(i), widths[i-1]);
          }
          kiir(s);
          if (figyu(rs.getWarnings())) rs.clearWarnings();        //adatveszts?
    }
    kiir("    --- Tbla vge ---");
    if (figyu(rs.getWarnings())) rs.clearWarnings();
  }

  public void actionPerformed(ActionEvent evt) {             //gombok lekezelse
    if (evt.getSource()==clearButton) {                   //eredmnymez trlse
       result.setText("");
       return;
    }

    if (evt.getSource()==registerButton) {       //meghajtprogram regisztrlsa
       try {
           Driver d=(Driver)Class.forName(driver.getText()).newInstance();
           kiir("Regisztrlt meghajtprogram: "+driver.getText());
           kiir("Verzi: "+d.getMajorVersion()+"."+d.getMinorVersion());
           String s;
           if (!d.jdbcCompliant()) s=" nem ";
           else s=" ";
           kiir("Ez a meghajtprogram"+s+"JDBC-megfelel.");
       } catch (Throwable e) {
         hiba("Nem sikerlt a regisztrci", e);
       }
    }

    if (evt.getSource()==connectButton) {          //kapcsolds az adatbzishoz
       try {
           String s;
           con=DriverManager.getConnection(url.getText(),
                          userid.getText(), new String(password.getPassword()));
           meta=con.getMetaData();
           kiir("Megnyitott adatbzis cme: "+meta.getURL());
           kiir("Felhasznl azonostja: "+meta.getUserName());
           kiir("Adatbzis tpusa: "+meta.getDatabaseProductName()+
                " "+meta.getDatabaseProductVersion());
           kiir("Felhasznlt meghajtprogram: "+meta.getDriverName()+
                " "+meta.getDriverVersion());
           if (figyu(con.getWarnings())) con.clearWarnings();
       } catch (Throwable e) {
         con=null;
         hiba("Nem sikerlt a kapcsolat megnyitsa", e);
       }
    }

    if (evt.getSource()==listButton) {                        //tblk listzsa
       if  (con==null) kiir("Nincs kapcsolat.");
       else try {
            kiir("Tblk listzsa:");
            tablakiir(con.getMetaData().getTables(null, null, null, null));
       } catch (Throwable e) {
         hiba("Nem sikerlt a tblk listzsa", e);
       }
    }
    
    if (evt.getSource()==execButton) {                        //SQL vgrehajtsa
       if  (con==null) kiir("Nincs kapcsolat.");
       else try {
           kiir("Vgrehajtand SQL: "+con.nativeSQL(sql.getText()));
           Statement stmt = null;
           try {                                                      //JDBC 2.0
               stmt = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE,
                                          ResultSet.CONCUR_UPDATABLE);
           } catch (Throwable e) {
                   stmt = con.createStatement();
           }
           boolean wasRS = stmt.execute(sql.getText());
           int rowCount;
           while (true)  {                    //eredmnykomponensek feldolgozsa
                 if (wasRS) {
                    ResultSet rs = stmt.getResultSet();
                    if (rs != null) {
                       tablakiir(rs);
                       wasRS = stmt.getMoreResults();
                       continue;
                    }
                    break;
                 }
                 rowCount = stmt.getUpdateCount();
                 if (rowCount >= 0) {
                    kiir("Megvltozott sorok szma = " + rowCount);
                    wasRS = stmt.getMoreResults();
                    continue;
                 }
                 break;
           }
           kiir("Vgrehajts befejezve.");
           if (figyu(stmt.getWarnings())) stmt.clearWarnings();
       } catch (Throwable e) {
         hiba("Nem sikerlt a vgrehajts", e);
       }
    }
    kiir("");                                                        //soremels
  }
}
