/* MegbizhatoEgyiranyuDatagramSocket.java
 *
 * Egy olyan UDP-alap kommunikcis vgpont, amely nyugtzssal s
 * szksg szerinti jrakldssel biztostja a "viszonylag"
 * megbzhat adattvitelt.
 *
 */

import java.net.*;
import java.io.*;
import java.lang.Math;

public class MegbizhatoEgyiranyuDatagramSocket
                extends FeledekenyDatagramSocket {

  private byte szmll=0; // az ppen elkldtt csomag sorszma
  private byte elzleg_rkezett_csomag_szmllja=0;
  private boolean tmout_belltva=false; // belltottuk mr az
                                         // idtllps figyelst?
  private boolean els_csomag=true; // fogadtunk mr csomagot ezen a
                                    // kommunikcis vgponton?

  public MegbizhatoEgyiranyuDatagramSocket() throws SocketException {
    super();
  }

  public MegbizhatoEgyiranyuDatagramSocket(int p) throws SocketException {
    super(p);
  }

  public MegbizhatoEgyiranyuDatagramSocket(int p, InetAddress la)
     throws SocketException {

    super(p,la);
  }

  private void inkrementl() {
    szmll=(byte)((szmll+1)%256); // 255-nl nullra forog t ...
  }

  private byte inkrementlt(byte b) {
    return (byte)((b+1)%256);
  }

  // Egy bjt mret segdadattal kiegsztett UDP-csomag elkldse
  // Lnyeg: az tkldeni kvnt csomag tartalmt egy j csomagba
  // msoljuk, s ell kiegsztjk az elkldeni kvnt bjttal.
  private void mb_send(DatagramPacket p, byte b) throws IOException {
    InetAddress ip_cm = p.getAddress();
    int server_port = p.getPort();
    int eredeti_zenet_hossz = p.getData().length;
    byte[] eredeti_zenet = p.getData();
    byte[] jzenet = new byte[eredeti_zenet_hossz+1];

    jzenet[0]=b; // kiegsztjk az elkldeni kvnt bjttal
    for (int i=0;i<eredeti_zenet_hossz;i++) {
      jzenet[i+1]=eredeti_zenet[i];
    }

    DatagramPacket sorszmozott_p=new DatagramPacket(jzenet,
                           eredeti_zenet_hossz + 1, ip_cm, server_port);
    super.send(sorszmozott_p); // tadjuk az UDP protokollnak
  }

  // Egy bjt mret segdadattal kiegsztett UDP-adatcsomag fogadsa.
  // Lnyege: a megrkezett csomagbl "kiszedjk" az els bjton lev
  // segdadatot, s az gy maradt csomagtartalmat adjuk vissza a
  // hvnak.
  private byte mb_receive(DatagramPacket p) throws IOException {
    byte[] hely = new byte[65536]; // ez a potencilisan legnagyobb
                                   // UDP-csomag mretnl is nagyobb
                         // de tkp. elg lenne p.getData().length is.
    DatagramPacket sorszmozott_p=new DatagramPacket(hely, 65536);
    super.receive(sorszmozott_p); // Fogadok egy UDP-csomagot
    if (sorszmozott_p.getLength() < 1) { // nulla hossz csomag nem j!
      super.receive(sorszmozott_p); // Fogadok egy jabb UDP-csomagot
    }
    p.setAddress(sorszmozott_p.getAddress());
    p.setPort(sorszmozott_p.getPort());

    byte[] eredeti_zenet = p.getData(); // Kinyerem az rkezett adatokat
    byte[] rkezett_zenet = sorszmozott_p.getData();
    // A hvnak az rkezett csomag tartalmt kell visszaadni, de ha
    // a hv ltal a metdus paramterben tadott csomag mrete
    // a szksgesnl rvidebb, akkor csak annyi karaktert adunk vissza.
    int msols_hossza = java.lang.Math.min(eredeti_zenet.length,
                                            rkezett_zenet.length);
    for (int i=0;i<msols_hossza;i++) {
      eredeti_zenet[i]=rkezett_zenet[i+1];
    }
    p.setLength(sorszmozott_p.getLength()-1);
    return rkezett_zenet[0];
  }

  // Elkld egy csomagot, s megvrja a cmzettl rkez nyugtt.
  public synchronized void nyugtzott_send(DatagramPacket p)
          throws IOException {

    inkrementl();   // Minden csomaggal j szmll rtket kldnk.
    byte zenet_szmll=szmll;
    byte visszakapott_rtk;
    boolean biztosan_megrkezett=false;
    byte[] tmp_buf=new byte[1];
    DatagramPacket q=new DatagramPacket(tmp_buf, 1); // Nyugtt fogadjuk

    if (!tmout_belltva) {
      setSoTimeout(500);    // Ha mg nem lltottuk be az adatcsomag
      tmout_belltva=true; // fogadsi idtllpsi hatrt, akkor most
    }                       // tesszk azt meg.
    while (!biztosan_megrkezett) { // Amg nem biztos, hogy trt ...
      mb_send(p,zenet_szmll);   // Elkldjk az aktulis sorszmmal
      // System.out.println("s:csomag elkldve ...");
      try {
        // System.out.println("s:nyugtra vrva  ...");
        visszakapott_rtk=mb_receive(q); // Vrunk a partner nyugtjra
        // System.out.println("s:kaptam nyugtt ...");
        if (zenet_szmll==visszakapott_rtk) { // J nyugta rkezett?
          biztosan_megrkezett=true;               // Igen, j nyugta ...
        }
      } catch (InterruptedIOException e) {
        // Idhatr tllpse esetn jra elkldjk
      }
      if (!biztosan_megrkezett) {
        // System.out.println("s:JRAKLDM...");
      }
    }
  }

  // Beolvassa a kvetkez csomagot, s nyugtt kld a feladnak.
  public synchronized void nyugtzott_receive(DatagramPacket p)
          throws IOException {

    byte visszakapott_rtk;
    boolean vanjcsomag=false;  // Az elz csomag msodpldnya?

    while (!vanjcsomag) {
      // Vrakozunk egy adatcsomagra
      visszakapott_rtk=mb_receive(p);
      // System.out.println("r:csomag rkezett...");
      // Kaptunk egy adatcsomagot
      if (els_csomag) {
        if (visszakapott_rtk == 1) { // mr az elejn 1-re nveltk
          vanjcsomag=true;            // A vrt csomag sorszmt elre
        }                              // ismerjk, nem szmolni kell.
        els_csomag=false;
        elzleg_rkezett_csomag_szmllja=visszakapott_rtk;
      } else {
        if (inkrementlt ( elzleg_rkezett_csomag_szmllja ) ==
                     visszakapott_rtk ) {

               // A fenti felttel vizsglata is elg ...
               vanjcsomag=true;       // Tnyleg ez a csomag kvetkezik...
               elzleg_rkezett_csomag_szmllja=visszakapott_rtk;
        } else {
               // System.out.println("Duplzott/nem a vrt csomag.");
        }
      }
      // Elksztnk egy nyugtt; oda, ahonnan a csomagot kaptuk
      byte[] tmp_buf=new byte[1];
      DatagramPacket q=new DatagramPacket(tmp_buf, 1, 
                                          p.getAddress(), p.getPort());
      // Visszakldjk a nyugtt
      // System.out.println("r:nyugtt kldk...");
      mb_send(q,visszakapott_rtk);
    }
  }
}
