/*
 * Sorozatkuld.java -- Megbzhat adatcsatornnkon csomagokat kld
 *
 * Ez az alkalmazs csomagokat kld az argumentumaiban megadott nev 
 * szmtgp megadott sorszm UDP-portjra. A csomagokban egy-egy sz
 * van, amiket a felhasznl a harmadik argumentumtl kezdve adhat meg.
 * Minden sz (pontosabban programargumentum) kln adatcsomagban kerl 
 * tkldsre.
 *
 * Programparamterek:
 *
 *  Els    : annak a hostnak a neve, ahol a szerver fut
 *  Msodik : a szerver UDP-portjnak a sorszma
 *  Harmadik, Negyedik ... : a szervernek egy-egy csomagban tkldend sz
 *
 * Egy plda a felhvsra:
 *
 *  java Sorozatkuld localhost 6789  szo1 szo2 ...
 *
 */

import java.io.*;
import java.net.*;

public class Sorozatkuld {

  public static void hasznlati_utasts() {
    System.out.println("Hasznlat: java Sorozatkuld " +
                       " szmtgpnv portsorszm tkldend_szavak");
    System.exit(-1);
  }

  public static void main(String[] args) {

    int server_port = 0;
    try {
      MegbizhatoEgyiranyuDatagramSocket s = 
          new MegbizhatoEgyiranyuDatagramSocket(); // j kommun. vgpont

      if (args.length < 3) hasznlati_utasts();
      try {
        server_port = Integer.parseInt(args[1]);
      } catch (NumberFormatException e) {
        hasznlati_utasts();
      }

      try {
        // A szerver cme
        InetAddress ip_cm = InetAddress.getByName(args[0]);  

        for (int i=2;i<args.length;i++) {

           int zenet_hossz = args[i].length(); // az zenet hossza
           byte[] zenet = args[i].getBytes();  // a csomag tartalma

           // az elkldend szt tartalmaz UDP-csomag objektum ltrehozsa
           DatagramPacket p = new DatagramPacket(zenet, zenet_hossz, 
                                               ip_cm, server_port); 

           s.nyugtzott_send(p); // elkldjk a csomagot
        }

      } catch (UnknownHostException e) {
        System.err.println("Ismeretlen nev szmtgp!");
      } catch (IOException e) {
        System.err.println("Nem sikerlt a csomagklds! "+e.getMessage());
      } finally {
        s.close(); // megszntetjk a kommunikcis vgpontunkat
      }
    } catch (SocketException e) {
      System.err.println("Az UDP-port lefoglalsa nem sikerlt!"); 
    }
  }
}
