import java.applet.Applet;                                  //appletrshoz kell
import java.awt.*;                             //grafikus komponensek miatt kell
import java.awt.event.*;                             //esemnykezels miatt kell

/**
 * LifeTest osztly - egy applet/alkalmazs letciklust szemllteti.
 */
public class LifeTest extends Applet implements ActionListener {
  final boolean DEBUG=false;                                 //nincs nyomkvets

  Panel buttonPanel=new Panel();                                //gombok panelje
  Button clearButton=new Button("clear");                          //nyomgombok
  Button validateButton=new Button("validate");
  Button resizeButton=new Button("resize");
  Button repaintButton=new Button("repaint");
  TextArea textArea=new TextArea(10, 80);                           //szvegmez

  static int instantCounter;                                   //pldnyszmll
  int instantID;                                                 //pldny szma
  int stepCounter;                                 //lthat kirt lpsek szma
  int blockTime=5;                               //blokkols hossza msodpercben

  boolean blockMain;                                        //main blokkoldik-e
  boolean blockInit;                                        //init blokkoldik-e
  boolean blockStart;                                      //start blokkoldik-e
  boolean blockStop;                                        //stop blokkoldik-e
  boolean blockDestroy;                                  //destroy blokkoldik-e
  boolean blockFinalize;                                //finalize blokkoldik-e
  boolean blockPaint;                                      //paint blokkoldik-e
  boolean blockUpdate;                                    //update blokkoldik-e
  boolean blockAction;                           //actionPerformed blokkoldik-e

  public LifeTest() {                                              //konstruktor
    super();
    instantID=++instantCounter;                         //pldnyszm eltrolsa

    buttonPanel.add(resizeButton);                          //fellet felptse
    resizeButton.addActionListener(this);                      //esemnyfigyels
    buttonPanel.add(validateButton);
    validateButton.addActionListener(this);
    buttonPanel.add(repaintButton);
    repaintButton.addActionListener(this);
    buttonPanel.add(clearButton);
    clearButton.addActionListener(this);
    textArea.setEditable(false);                    //szvegmez nem mdosthat
    setLayout(new BorderLayout());                 //pakolsi stratgia megadsa
    add("North", buttonPanel);                             //gombok fnt lesznek
    add("Center", textArea);                   //tbbi terleten szvegmez lesz

    validate();                                 //grafikus fellet aktualizlsa
    addItem("konstruktor");                              //konstruktor listzsa
  }

  public static void main(String args[]) {                  //alkalmazs indts
    LifeTest myLifeTest=new LifeTest();                          //pldnyosts
    Frame myFrame=new Frame(myLifeTest.getClass().getName()+" alkalmazs");
    myFrame.addWindowListener(
        new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                   System.exit(0);                        //kilps a programbl
            }
        }
    );
    myFrame.add("Center", myLifeTest);                      //fellet felptse
    if (args.length>0) {                              //paramterek feldolgozsa
       String param=args[0];
       int i;
       try {
           i=1;                                       //els a blokkols hossza?
           myLifeTest.blockTime=Integer.parseInt(param);
       } catch (Exception e) {
         i=0;
       }
       param="";
       while (i<args.length) param+=args[i++];
       myLifeTest.useParams(param);                  //metdusnevek feldolgozsa
    }
    myFrame.pack();
    myFrame.show();                                        //ablak megjelentse
    myLifeTest.addItem("main");                               //main kilistzsa
    myLifeTest.block(myLifeTest.blockMain);
  }


  public void init() {                                      //indts appletknt
    String param;

    if ((param=getParameter("blockTime"))!=null)      //paramterek feldolgozsa
       try {
           blockTime=Integer.parseInt(param);
       } catch (Exception e) {}
    if ((param=getParameter("blockMethods"))!=null) useParams(param);
    addItem("init");                                          //init kilistzsa
    block(blockInit);
  }

  public void start() {
    addItem("start");                                        //start kilistzsa
    block(blockStart);
  }
  public void stop() {
    addItem("stop");                                          //stop kilistzsa
    block(blockStop);
  }

  public void destroy() {
    addItem("destroy");                                      //destroy listzsa
    block(blockDestroy);
  }

  public void finalize() {
    addItem("finalize");                                      //finalize kirsa
    block(blockFinalize);
  }

  public void paint(Graphics g) {
    addItem("paint");                                        //paint kilistzsa
    block(blockPaint);
    super.paint(g);
  }
  public void update(Graphics g) {
    addItem("update");                                      //update kilistzsa
    block(blockUpdate);
    super.update(g);
  }

  public void actionPerformed(ActionEvent evt) {                //esemnykezels
    if (DEBUG) System.out.println(instantID+". pldny action: "+evt);
    addItem("actionPerformed");                    //actionPerformed kilistzsa
    block(blockAction);
    if (evt.getSource()==clearButton) {                                  //clear
           textArea.setText("");
           stepCounter=0;
    } else if (evt.getSource()==repaintButton) {                       //repaint
           repaint();
    } else if (evt.getSource()==validateButton) {                     //validate
           validate();
    } else if (evt.getSource()==resizeButton) {                         //resize
           resize(getSize().height, getSize().width);
    }
  }

  private void useParams(String param) {                 //paramter-feldolgozs
    if (DEBUG) System.out.println(instantID+". pldny param: "+param);
    if (blockTime<0) blockTime=0;
    param=param.toLowerCase();
    blockMain=param.indexOf("main")!=-1; blockInit=param.indexOf("init")!=-1;
    blockStart=param.indexOf("start")!=-1; blockStop=param.indexOf("stop")!=-1;
    blockDestroy=param.indexOf("destroy")!=-1;
    blockFinalize=param.indexOf("finalize")!=-1;
    blockPaint=param.indexOf("paint")!=-1;
    blockUpdate=param.indexOf("update")!=-1;    
    blockAction=param.indexOf("action")!=-1;
  }

  private void addItem(String s) {                         //metdus kilistzsa
    s=instantID+". pldny "+(++stepCounter)+". lps: "+s+" meghvsa a "+
      Thread.currentThread()+Thread.currentThread().getThreadGroup()+"-bl.\n";
    System.out.print(s);
    textArea.append(s);
  }

  private void block(boolean blocking) {                             //blokkols
    if (blocking) {
       long blockEndTime=System.currentTimeMillis()+1000;
       int blockCounter=blockTime;
       while (blockCounter-->0) {                       //msodpercek szmllsa
             if (DEBUG) 
                System.out.println(instantID+". blokkolva:"+blockCounter);
             while (System.currentTimeMillis()<blockEndTime);
             blockEndTime+=1000;
         }
    }
  }
}
