import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;

/**
 * AShape class is a JComponent, which contains a RectangularShape
 * as its only displayable part.
 */
abstract public class AShape extends JComponent {
    protected RectangularShape    shape;
    protected Rectangle2D        shapeBounds;
    
    public AShape(RectangularShape theShape) {
        shape = theShape;
        shapeBounds = shape.getBounds2D();

        Rectangle intBounds = shape.getBounds();
        super.setBounds(intBounds.x, intBounds.y,
            intBounds.width + 1, intBounds.height + 1);

        Dimension size = new Dimension(intBounds.width, intBounds.height);
        super.setMinimumSize(size);
        super.setPreferredSize(size);

        repaint();
    }

    public boolean contains(int x, int y) {
        return shape.contains(
            shapeBounds.getMinX() + x, 
            shapeBounds.getMinY() + y
        );
    }

    public void reshape(int x, int y, int w, int h) {
        super.reshape(x, y, w, h);
        ((RectangularShape)shape).setFrame(x, y, w - 1, h - 1);
        shapeBounds = shape.getBounds2D();
        repaint();
    }

    // It assumes that the passed Graphics is an instance of Graphics 2D!
    protected void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D)g;
        
        g2.setColor(getForeground());
        g2.setBackground(getBackground());

        g2.translate(-shapeBounds.getMinX(), -shapeBounds.getMinY());
        if(!isOpaque()) {
            g2.fill(shape);
        }
        g2.draw(shape);
    }

} // AShape

