import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.lang.reflect.*;

public class Shapes extends JFrame {
    private JPanel        canvas;
    private JTextField    sName;
    private JTextField    sX;
    private JTextField    sY;
    private JTextField    sWidth;
    private JTextField    sHeight;

    public Shapes() {
        getContentPane().setLayout(new BorderLayout());
        createCanvas();
        createUI();
        createMenu();
    }

    protected void createCanvas() {
        canvas = new JPanel();
        canvas.setSize(new Dimension(500, 400));
        canvas.setLayout(new CanvasLayout());
        canvas.setMinimumSize(new Dimension(500, 400));
        canvas.setPreferredSize(new Dimension(600, 500));


        getContentPane().add(canvas, BorderLayout.CENTER);
        canvas.repaint();
    }

    public void createUI() {
        JPanel ui = new JPanel();
        ui.add(new JLabel("Name:"));
        sName = new JTextField(10); ui.add(sName);
        ui.add(new JLabel("x:"));
        sX = new JTextField(3); ui.add(sX);
        ui.add(new JLabel("y:"));
        sY = new JTextField(3); ui.add(sY);
        ui.add(new JLabel("width:"));
        sWidth = new JTextField(3); ui.add(sWidth);
        ui.add(new JLabel("height:"));
        sHeight = new JTextField(3); ui.add(sHeight);
        
        ClearAction ca = new ClearAction();
        JButton create = new JButton("Create"); ui.add(create);
        create.setDefaultCapable(true);
        create.addActionListener(ca);
        create.addActionListener(new CreateAction());

        JButton clear = new JButton("Clear"); ui.add(clear);
        clear.addActionListener(ca);
        
        getContentPane().add(ui, BorderLayout.SOUTH);
    }

    protected void createMenu() {
        JMenuBar menuBar = new JMenuBar();
        JMenu menu = new JMenu("File");
        menuBar.add(menu);
        setJMenuBar(menuBar);

        menu.add(new DumpAction("dump shapes"));

        menu.addSeparator();

        menu.add(new AbstractAction("Exit") {
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        });
    }


    public static void main(String[] args) {
        Shapes frame = new Shapes();

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

        frame.pack();
        frame.setVisible(true);
    }

    protected class ClearAction extends AbstractAction {
        public void actionPerformed(ActionEvent e) {
            sName.setText("");
            sX.setText("");
            sY.setText("");
            sWidth.setText("");
            sHeight.setText("");
        }
    } // ClearAction

    protected class CreateAction extends AbstractAction {
        protected JComponent createShape(String name) {
            JComponent s = null;
            try {
                Class shapeClass = Class.forName(name);
                Constructor shapeConstructor =
                    shapeClass.getConstructor(null);
                s = (JComponent)shapeConstructor.
                    newInstance(null);
            }
            catch(Exception e) {
                System.err.println(e);
            }
            return s;
        }

        protected void reshapeShape(Object s, int x, int y, int w, int h) {
            try {
                Class shapeClass = s.getClass();
                Class parTypes[] = {Integer.TYPE, Integer.TYPE,
                    Integer.TYPE, Integer.TYPE};
                Method reshapeMethod = shapeClass.getMethod(
                    "reshape", parTypes);
                Object parameters[] = {new Integer(x), new
                    Integer(y), new Integer(w), new Integer(h)};
                reshapeMethod.invoke(s, parameters);
            }
            catch(Exception e) {
                System.err.println(e);
            }
        }

        public void actionPerformed(ActionEvent e) {
            String name = sName.getText();
            int x, y, width, height;
            try {
                x = Integer.parseInt(sX.getText());
                y = Integer.parseInt(sY.getText());
                width = Integer.parseInt(sWidth.getText());
                height = Integer.parseInt(sHeight.getText());
            } 
            catch(NumberFormatException ex) {
                System.err.println("please numbers!");
                return;
            }
            if(x < 0) x = 0; if(x > 1000) x = 1000;
            if(y < 0) y = 0; if(y > 500) y = 500;
            if(width < 10) width = 10;
            if(width > 1000) width = 1000;
            if(height < 10) height = 10;
            if(height > 500) height = 500;
            
            JComponent s = createShape(name);
            if(s == null) return;
            reshapeShape(s, x, y, width, height);

            s.setToolTipText(name);
            canvas.add(s);
            canvas.repaint();
        }
    } // CreateAction

    protected class DumpAction extends AbstractAction {
        public DumpAction(String name) {
            super(name);
        }
        
        public void actionPerformed(ActionEvent e) {
            System.out.println("Classes:");
            Component cs[] = canvas.getComponents();
            for(int i = 0; i < cs.length; i++) {
                Class cClass = cs[i].getClass();
                System.out.println("  " + cClass.getName());
            }
        }
    } // DumpAction
} // Shapes

