// FoglaloOsztaly.java
//
// Egy tranzakciban egy adott idpontra bejegyez egy
// elfoglaltsgot nhny (tbb) naptrba
// A main() metdusban ltrehozunk egy pldnyt az osztlybl, s
// bejegyezzk az RMI nvszolgaltatba "FoglaloOsztaly" nven.

import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.Naming;
import java.rmi.server.*;

public class FoglaloOsztaly extends UnicastRemoteObject 
  implements FoglaloInterface {

  // A konstruktor "exportlja" az objektumot ...
  public FoglaloOsztaly() throws java.rmi.RemoteException {
    super();
  }

  // Egssz konvertlja a paramterben szveges formaban
  // megadott szmot, majd visszakonvertlja szvegesre
  // Ezzel megoldhat, hogy pl. 005 s 5 egyformn legyenekkezelve.
  public static String Egeszre(String s) throws NumberFormatException {
    return (new Integer(Integer.parseInt(s))).toString();
  }

  // Egy tranzakciban egy adott idpontra bejegyez egy
  // elfoglaltsgot nhny (tbb) naptrba
  //
  // A paramterben egy szveges vektort vr, a tartalmt a
  // kvetkezk szerint kell sszelltani:
  // args[0],args[1],args[2]: a bejegyzend elfoglaltsg idpontja
  //          (v/hnap/nap, tzes szmrendszerbeli szmok)
  // args[3]: a bejegyzend elfoglaltsg lersa
  // args[4], ... : azoknak a szemlyeknek az azonostja, akiknek a
  //           naptrba be akarjuk jegyezni az elfoglaltsgot
  // Visszatrsi rtkben jelzi a tranzakci sikeressgt/abortlst
  public boolean Elfoglaltsag_Bejegyzese(String args[]) 
    throws TranzakcioException, java.rmi.RemoteException {

    String mi; // a bejegyezni kvnt elfoglaltsg lersa
    String mikor; // a bejegyzend elfoglaltsg idpontja
    boolean tranzakciokezelo_letrehozva=false;

    try {
      // Elrjk a tranzakci szolgltatst
      TranzakcioManagerFactoryInterface tf=
              (TranzakcioManagerFactoryInterface)
      Naming.lookup("RMI_tranzakciok");

      // Ltrehozunk egy j tranzakcit
      TranzakcioManagerInterface ti=(TranzakcioManagerInterface)
              tf.Ujtranzakcio();
      tranzakciokezelo_letrehozva=true;

      mikor=new String(Egeszre(args[0])+"/"+
                       Egeszre(args[1])+"/"+
                       Egeszre(args[2]));
      mi=args[3];
      for (int i=0;i<args.length-4;i++) {
        // Megszerznk egy referencit a kvetkez
        // rsztvev naptrhoz
        try {
          NaptarInterface bsz=(NaptarInterface) Naming.lookup(args[i+4]);
          bsz.Lefoglal(mikor,mi,ti); // Naptrba bejegyzs
        } catch (java.rmi.RemoteException e) {
          ti.Abortal();
          throw new TranzakcioException("Nem tudtam elrni valamelyik"+
                        " rsztvevt, tranzakci fggben marad!");
        }
      }

      // Megkrjk a vezrt (koordintort) a tranzakci
      // levezetsre, majd kirjuk a kimenetelt.
      return ti.Dontes();
    } catch (java.net.MalformedURLException e) {
      throw new TranzakcioException("A megadott rsztvev neve rvnytelen"+
                                    " formj!");
    } catch (java.rmi.NotBoundException e) {
      throw new TranzakcioException("A futshoz szksges valamely"+
                                    " szolgltats nem elrhet!");
    } catch (java.rmi.UnknownHostException e) {
      throw new TranzakcioException("A futshoz szksges valamely"+
                                    " szolgltats nem elrhet!");
    } catch (java.rmi.RemoteException e) {
      throw new TranzakcioException("Nem tudtam ltrehozni egy tranzakci"+
                                    " koordintort!");
    }
  }


  public static void main(String args[]) {

    System.setSecurityManager(new RMISecurityManager());

    // Ltrehozunk egy pldnyt az osztlyunkbl, s
    // bejegyezzk a nvszolgltatba "FoglaloOsztaly" nven.
    try {
      FoglaloOsztaly fo = new FoglaloOsztaly();
      Naming.rebind("FoglaloOsztaly",fo);
      System.out.println("Foglal osztly fut!");
    } catch (Exception e) {
      System.out.println("Foglal osztly nem fut!"+e);
    }
  }
}
