// Naptar.java
//
// Egy naptr feladatait megvalst osztly.
// Fprogramja ltrehoz egy j res naptr objektumot, s bejegyzi 
// az RMI nvszolgltatjba. A naptr tulajdonosnak a nevt az egyetlen
// programparamterben vrja, ezen a nven jegyzi be a naptr objektumot a
// nvszolgltatba.

import java.rmi.server.UnicastRemoteObject;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.util.Vector;
import java.util.Hashtable; 

public class Naptar extends UnicastRemoteObject 
  implements NaptarInterface, TranzakcioResztvevoInterface {

  String tulajneve; // A naptr tulajdonosnak neve
  Hashtable elfoglaltsag; // (DatumString, String)
  Hashtable visszacsinal; // (Tid, Vektor(DatumString)) prok; ezek jelzik, 
                          // hogy mikor is mdostott az illet tranzakci
  Hashtable kommittalhat; // (Tid, Valami) pr, amelyik tranzakci mr
                          // biztosan nem vglegesthet.
  private static Object mutex = new Object(); // Klcsns kizrshoz

  // A konstruktor mvelet ltrehoz egy res naptrat
  public Naptar(String tulajneve) throws RemoteException {
    super(); // FONTOS, ezzel lesz tvolrl elrhet az objektum
    this.tulajneve=tulajneve;
    this.elfoglaltsag=new Hashtable();
    this.visszacsinal=new Hashtable();
    this.kommittalhat=new Hashtable();
    // res naptr elkszlt ...
  }

  // Ezzel lehet egy adott idpontra egy elfoglaltsgot bejegyezni
  public void Lefoglal(String mikor, String mi,
                       TranzakcioManagerInterface ti) 
                       throws java.rmi.RemoteException {

    // rtesti a koordintort, hogy ebben a tranzakciban
    // rsztvevknt tartsa nyilvn ezt az objektumot is.
    ti.Tranzakcio_Resztvevo_Bejegyzese(tulajneve,this);
    // Klcsnsen kizrjuk az esetleges prhuzamos mdostsokat
    synchronized (mutex) {
      // Ha errl a tranzakcirl mg nem trolunk adatokat,
      // akkor ltrehozzuk a szksges llapothordozkat.
      if (!(visszacsinal.containsKey(ti.getTid()))) {
        Vector umodositasok=new Vector();
        visszacsinal.put(ti.getTid(), umodositasok);
        umodositasok=null;
      }

      // Megnzzk, hogy szabadok vagyunk-e mg a
      // a krdses napon.
      if (elfoglaltsag.containsKey(mikor)) {
        // Ha a krdses nap mr foglalt, akkor nem 
        // jegyznk be semmit, hanem
        // abortljuk a tranzakcit (MAJD)
        // (Mindegy mit trolunk a 2. paramterben!)
        kommittalhat.put(ti.getTid(),mikor); 
        return; // vissza a hvhoz ...
      } 

      // Eltroljuk, hogy mikorra jegyzett be elfoglaltsgot
      // az illet tranzakci.
      Vector modositasok=(Vector) visszacsinal.get(ti.getTid());
      modositasok.insertElementAt(mikor,modositasok.size());
      // s be is jegyezzk, nehogy msvalaki bejegyezzen
      // valami ms elfoglaltsgot.
      elfoglaltsag.put(mikor,mi);
      System.out.println("Lefoglalsi ksrlet:"+mikor.toString()+
                         "-kor elfoglaltsg:"+mi);
    }
  }

  // Egy abortlt tranzakci tevkenysgeit semlegestjk
  private void Visszacsinal(String tid) {
    int index;

    // Megszerezzk a mdostsok listjt
    Vector modositasok=(Vector) visszacsinal.get(tid);
    index=0;
    for (index=0;index<modositasok.size();index=index+1) {
      // Kinyerjk a mdosts dtumt
      String o=(String) (modositasok.elementAt(index));
      // Trljk az elfoglaltsgok kzl
      elfoglaltsag.remove(o);
    }
  }

  // Itt kvetkeznek a ktfzis megegyezst tmogat metdusok

  // Els fzis: a tranzakci rm es rszt tudom-e vglegesteni?
  public boolean Felkeszul(String tid) throws java.rmi.RemoteException {
    if (kommittalhat.containsKey(tid)) {
      return false; // Nem vglegesthet (ui. tkztt valamivel)
    } else {
      return true; // Nem volt tkzs, vglegesthet
    }
  }

  // Msodik fzis: a tranzakci rm es rszt vglegesteni kell vagy
  // vissza kell csinlni ... (b=igaz/hamis attl fggen, mi is a teend)
  public void Veglegesit(boolean b, String tid)
    throws java.rmi.RemoteException {

    if (!b) {
      Visszacsinal(tid); // Nem vglegesthet ...
    }
    // Trljk a tranzakcirl trolt dolgokat, mr nem kellenek
    visszacsinal.remove(tid);
    kommittalhat.remove(tid);
    System.out.println(tid+" tranzakci lezrva.");
  }
	
  // Ktfzis megegyezst tmogat metdusok vge

  public static void main(String args[]) {
    System.setSecurityManager(new RMISecurityManager());
    try {
      // Ltrehozunk egy j res naptr objektumot
      Naptar n = new Naptar(args[0]);
      // Az j naptrat bejegyezzk az args[0]-ban
      // megadott nven a nvszolgltatba
      // Konvenci: args[0] a naptr tulajdonosnak a neve
      Naming.rebind(args[0],n);
      System.out.println(args[0]+" naptra aktv!");
    } catch (Exception e) {
      System.out.println(args[0]+" naptrat nem lehetett ltrehozni!");
    }
  }
}
