// RMIMutex.java
//
// Ez egy tavolrol elerheto Mutex

import java.rmi.Naming;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;

public class RMIMutex extends UnicastRemoteObject 
      implements RMIMutexInterface {

  public Mutex localMutex; // Ezt a mutexet tesszuk tavolrol elerhetove

  public RMIMutex() throws RemoteException {
    super(); // Tvoli (konstruktor) objektum exportlasa
    localMutex = new Mutex(); // Letrehozzuk a tavolrol elerheto mutexet
  }

  // A mutex "lefoglal" muvelete. A folyamatok a kritikus szakaszba lepes
  // elott foglaljak le a mutexet.
  // Varakozik, amig nem lephet be a kritikus szakaszba.
  // Kivaltahato kivetelek:
  //   java.rmi.RemoteException : ha a tavoli metodushivasi rendszer hibat eszlel
  //   java.lang.InterruptedException : a mutex "lefoglalasa" sikertelen, mert
  //                a wait() muveletnel valo varakozast valamilyen esemeny
  //                megszakitotta. Ilyenkor erdemes ujra meghivni a Lefoglal()
  //                tavoli metodust.
  public void Lefoglal() throws java.rmi.RemoteException, 
                                java.lang.InterruptedException {
    localMutex.Lefoglal();
  }

  // A mutex korlatozott ideig varakozo "lefoglal" muvelete. 
  // A folyamatok a kritikus szakaszba lepes elott foglaljak le a mutexet.
  // Varakozik, amig nem lephet be a kritikus szakaszba, vagy a metodus
  // parametereben ezredmasodpercekben megadott varakozasi ido le nem telik.
  // Kivaltahato kivetelek:
  //   java.rmi.RemoteException : ha a tavoli metodushivasi rendszer hibat eszlel
  //   java.lang.InterruptedException : a mutex "lefoglalasa" sikertelen, mert
  //                a wait() muveletnel valo varakozast valamilyen esemeny
  //                megszakitotta. Ilyenkor erdemes ujra meghivni a Lefoglal()
  //                tavoli metodust.
  //   Timeout : letelt a parameterben megadott varakozasi ido, es ezalatt
  //                nem sikerult a mutex lefoglalasa.
  public void Korlatozottlefoglal(long ms) 
    throws java.rmi.RemoteException, java.lang.InterruptedException, Timeout {

    localMutex.Korlatozottlefoglal(ms);
  }

  // Egy "lefoglalt" mutex felszabaditasat vegzo muvelet.
  // Nem ellenorzi, hogy a megadott mutex tenyleg "foglalt" allapotban van-e;
  // de egy robusztusabb implementacioban ezt meg lehetne tenni.
  public void Felszabadit() throws java.rmi.RemoteException, 
                                   java.lang.InterruptedException {
    localMutex.Felszabadit();
  }

}
