// TranzakcioManager.java
//
// A ktfzis megegyezsi protokollt megvalst osztly.
// Ezen osztly objektumai kpesek egy-egy tranzakci leveznylsre.

import java.rmi.Naming;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.util.Hashtable;
import java.util.Enumeration;

public class TranzakcioManager extends UnicastRemoteObject 
  implements TranzakcioManagerInterface {

  String nev; // A tranzakci - szveges - azonostjt trolja
  Hashtable rv; // A tranzakciban rsztvevk identitst trolja
  boolean koordinator_dontese=true; // Feltesszk, hogy mindenkinl OK.
  boolean dontottem=false; // Kls helyrl mg abortlhatjk
  Object koordinator_dontese_mutex = new Object(); // Klcsns kizrshoz

  public TranzakcioManager(String nev) throws RemoteException {
    super();
    this.nev=nev;
    rv=new Hashtable();
  }

  // Egy - esetleg j - rsztvev feljegyzse ...
  // Az "rvneve" paramter kulcs, teht ha egy rsztvev ugyanazon a
  // nven ktszer is bejegyezteti magt, az nem okozhat problmt
  // (az elz bejegyzs egyszeren "nmagval" fell lesz rva)
  // Az objektumreferencia hash-elsvel az ilyen problmk is
  // kiszrhetek lennnek, de most ezzel inkbb nem foglalkozunk.
  public void Tranzakcio_Resztvevo_Bejegyzese(String rvneve, 
            TranzakcioResztvevoInterface kit) 
            throws RemoteException {

    synchronized (rv) {
      rv.put(rvneve, kit); // Szveges azonostt+referencit
    }
  }

  // Egy kls (nem kzvetlen rsztvev) alkalmazs (pl. a kezdemnyez)
  // krheti a tranzakci abortlst
  public void Abortal() throws TranzakcioException, RemoteException {
    synchronized (koordinator_dontese_mutex) {
      if (dontottem) {
        throw new TranzakcioException("Akkor prbltak abortlni, amikor "+
                "a vgeredmny mr nem befolysolhat!");
      }
      koordinator_dontese=false;
    }
  }

  // Itt trtnik a ktfzis megegyezs ...
  public boolean Dontes() throws TranzakcioException, RemoteException {
    Enumeration resztvevok=rv.elements();
    boolean b=false;

    // Vgigkrdezi a rsztvevket, hogy felkszltek-e a
    // belltsok vglegestsre
    while (resztvevok.hasMoreElements()) {
      System.out.println("Kvetkez rsztvev megkredezse ("+
                         this.nev+") ...");
      TranzakcioResztvevoInterface o=
          (TranzakcioResztvevoInterface) resztvevok.nextElement();
      try {
        b=o.Felkeszul(this.nev); // Felkszlt?
      } catch (java.rmi.RemoteException e) {
        synchronized (koordinator_dontese_mutex) {
          koordinator_dontese=false; // ABORT
        }
      }
      synchronized (koordinator_dontese_mutex) {
        koordinator_dontese=(koordinator_dontese && b);
      }
    }

    // Itt mr megvan a dnts a tranzakci kimenetelrl

    // rtestjk a rsztvevket a tranzakci kimenetelrl
    synchronized (koordinator_dontese_mutex) {
      dontottem=true;
    }
    if (koordinator_dontese) {
      System.out.println("A "+this.nev+" tranzakci vglegesthet.");
    } else {
      System.out.println("A "+this.nev+" tranzakci abortlva lesz.");
    }
    resztvevok=rv.elements();
    while (resztvevok.hasMoreElements()) {
      System.out.println("Kvetkez rsztvev rtestse ("+
                         this.nev+") ...");
      TranzakcioResztvevoInterface o=
          (TranzakcioResztvevoInterface) resztvevok.nextElement();
      try {
        o.Veglegesit(koordinator_dontese,this.nev);
      } catch (java.rmi.RemoteException e) {
        throw new TranzakcioException("Tranzakcio fuggoben maradt!");
      }
    }
    // A hvnak is visszaadjuk a vgeredmnyt
    return koordinator_dontese; //  Nem kell szinkronizlni ...
  }

  // Visszaadja a tranzakci szveges azonostjt
  public String getTid() throws RemoteException {
    return this.nev;
  }

}
