/******************************************************************************
 * Applet: MyLittleApplet
 * Purpose: The applet is created for the demostration of security features of
 * the Java 2 platform related to signed applets.
 * Created by Gbor Pcsy (pici@elte.hu)
 * Creation date: 14:46PM  01.02.2001
 * Last modified: 14:46PM  01.02.2001
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class MyLittleApplet extends Applet implements ActionListener {

Panel buttonPanel       = null;
Button newButton        = null;
Button loadButton       = null;
Button saveButton       = null;
Button clearButton      = null;
Button quitButton       = null;
TextArea fileContent    = null;
TextField fileName      = null;

private final int MAXBUFFSIZE = 1000000;

public MyLittleApplet() {
    super();
    createMyLittleApplet(false);
} // MyLittleApplet

public MyLittleApplet(boolean needQuit) {
    super();
    createMyLittleApplet(needQuit);
} // MyLittleApplet

private void createMyLittleApplet(boolean needQuit) {
    buttonPanel = new Panel();
    newButton   = new Button("New");
    loadButton  = new Button("Load");
    saveButton  = new Button("Save");
    clearButton = new Button("Clear");
    fileContent = new TextArea("",10,80,TextArea.SCROLLBARS_BOTH);
    fileName    = new TextField("",80);

    buttonPanel.add(newButton);
    newButton.addActionListener(this);
    buttonPanel.add(loadButton);
    loadButton.addActionListener(this);
    buttonPanel.add(saveButton);
    saveButton.addActionListener(this);
    saveButton.setEnabled(false);
    buttonPanel.add(clearButton);
    clearButton.addActionListener(this);
    clearButton.setEnabled(false);
    if( needQuit ) {
        quitButton  = new Button("Quit");
        buttonPanel.add(quitButton);
        quitButton.addActionListener(this);
    }
    fileContent.setEditable(false);
    fileName.setEditable(true);
    setLayout(new BorderLayout());
    add("North",fileName);
    add("Center",buttonPanel);
    add("South",fileContent);

    validate();
} // createMyLittleApplet

public static void main(String args[]) {
    MyLittleApplet application  = new MyLittleApplet(true);
    Frame myFrame               = new Frame(application.getClass().getName()+
                                            " application");
    myFrame.addWindowListener( new WindowAdapter() {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    });
    myFrame.add("Center", application);

    myFrame.pack();
    myFrame.show();
} // main

private void loadFile() throws IOException, FileNotFoundException {
    File inFile = new File(fileName.getText());
    try {
        if( inFile.canRead() ) {
            FileInputStream fis     = new FileInputStream(inFile);

            if( fis.available() > MAXBUFFSIZE ) {
                fileContent.setText("File is too big ("+fis.available()+
                                    "bytes) and does not fit into memory!");
            } else {
                byte buff[] = new byte [MAXBUFFSIZE];
                int  len    = 0;

                len = fis.read(buff);
                fileContent.setText(new String(buff,0,len));
                fileContent.setEditable(true);
                fileName.setEditable(false);
                saveButton.setEnabled(true);
                clearButton.setEnabled(true);
                loadButton.setEnabled(false);
                newButton.setEnabled(false);
                fis.close();
            }

        } else {
            fileContent.setText(fileName.getText()+" does not exist!");
        }
    } catch (SecurityException e) {
        fileContent.setText("Exception thrown: "+e);
    }
    fileContent.requestFocus();
    validate();
} // loadFile

private void saveFile() throws IOException, FileNotFoundException {
    File outFile            = new File(fileName.getText());
    FileOutputStream fos    = null;
    try {
        if( outFile.canWrite() ) {
           fos = new FileOutputStream(outFile);
        } else {
            try {
                outFile.createNewFile();
                fos = new FileOutputStream(outFile);
            } catch(SecurityException e) {
                fileContent.setText("SecurityException thrown: "+e);
            }
        }

        if (fos != null ) {
            fos.write(fileContent.getText().getBytes());
            fos.close();
        }
    } catch (SecurityException e) {
        fileContent.setText("SecurityException thrown: "+e);
    }

    fileContent.requestFocus();
    validate();
} // saveFile

private void clearWindow() {
    fileName.setText("");
    fileName.setEditable(true);
    fileName.requestFocus();
    fileContent.setText("");
    fileContent.setEditable(false);

    saveButton.setEnabled(false);
    clearButton.setEnabled(false);
    loadButton.setEnabled(true);
    newButton.setEnabled(true);

    validate();
}

private void newFile() {
    if( fileName.getText().length() > 0 ) {
        fileContent.setText("");
        fileContent.setEditable(true);
        fileName.setEditable(false);
        saveButton.setEnabled(true);
        clearButton.setEnabled(true);
        loadButton.setEnabled(false);
        newButton.setEnabled(false);
        fileContent.requestFocus();
    } else {
        fileContent.setText("Specify a file name first!");
        fileName.requestFocus();
    }
    validate();
} // newFile

public void actionPerformed(ActionEvent e) {
    try {
        if( e.getSource()==loadButton ) {
            loadFile();
        } else if( e.getSource()== newButton ) {
            newFile();
        } else if( e.getSource()== saveButton ) {
            saveFile();
        } else if( e.getSource()== clearButton ) {
            clearWindow();
        } else if( e.getSource()== quitButton ) {
            System.exit(0);
        }
    } catch (Exception exc) {
        System.err.print("Unhandled exception:" + exc +"\n" );
    }
}

public void init() {

} // init

public void start() {
} // start

public void stop() {
} // stop

public void destroy() {
} // destroy

} // End of class MyLittleApplet